# -*- coding:utf-8 -*-
from mako import runtime, filters, cache
UNDEFINED = runtime.UNDEFINED
__M_dict_builtin = dict
__M_locals_builtin = locals
_magic_number = 9
_modified_time = 1422216484.887167
_enable_loop = True
_template_filename = '/usr/lib/python2.7/dist-packages/privacyidea/lib/tokens/emailtoken.mako'
_template_uri = '/emailtoken.mako'
_source_encoding = 'utf-8'
from webhelpers.html import escape
_exports = []


def render_body(context,**pageargs):
    __M_caller = context.caller_stack._push_frame()
    try:
        __M_locals = __M_dict_builtin(pageargs=pageargs)
        c = context.get('c', UNDEFINED)
        _ = context.get('_', UNDEFINED)
        __M_writer = context.writer()
        # SOURCE LINE 2
        __M_writer(u'\n')
        # SOURCE LINE 3
        if c.scope == 'config.title' :
            # SOURCE LINE 4
            __M_writer(escape(_("E-mail OTP token")))
            __M_writer(u'\n')
        # SOURCE LINE 6
        __M_writer(u'\n\n')
        # SOURCE LINE 8
        if c.scope == 'config' :
            # SOURCE LINE 9
            __M_writer(u'<!-- #################### E-mail provider ################### -->\n<script>\n/*\n * \'typ\'_get_config_val()\n *\n * this method is called, when the token config dialog is opened\n * - it contains the mapping of config entries to the form id\n * - according to the Config entries, the form entries will be filled\n *\n */\nfunction email_get_config_val(){\n\tvar ret = {};\n\tret[\'EmailProvider\'] = \'c_email_provider\';\n\tret[\'EmailProviderConfig\'] = \'c_email_provider_config\';\n\tret[\'EmailChallengeValidityTime\'] = \'c_email_challenge_validity\';\n\tret[\'EmailBlockingTimeout\'] = \'c_email_blocking\';\n\treturn ret;\n}\n/*\n * \'typ\'_get_config_params()\n *\n * this method is called, when the token config is submitted\n * - it will return a hash of parameters for system/setConfig call\n *\n */\nfunction email_get_config_params(){\n\tvar ret = {};\n\tret[\'EmailProvider\'] = $(\'#c_email_provider\').val();\n\tret[\'EmailProviderConfig\'] = $(\'#c_email_provider_config\').val();\n\tret[\'EmailProviderConfig.type\'] = \'password\';\n\tret[\'EmailChallengeValidityTime\'] = $(\'#c_email_challenge_validity\').val();\n\tret[\'EmailBlockingTimeout\'] = $(\'#c_email_blocking\').val();\n\treturn ret;\n}\n\n$(document).ready(function () {\n    $("#form_emailconfig").validate();\n});\n</script>\n\n<form class="cmxform" id="form_emailconfig">\n<fieldset>\n    <legend>')
            # SOURCE LINE 51
            __M_writer(escape(_("E-mail provider config")))
            __M_writer(u'</legend>\n    <table>\n        <tr>\n\t        <td><label for="c_email_provider">')
            # SOURCE LINE 54
            __M_writer(escape(_("E-mail provider")))
            __M_writer(u'</label>: </td>\n\t        <td><input type="text" name="email_provider" class="required"  id="c_email_provider" size="37" maxlength="80"></td>\n        </tr>\n        <tr>\n\t        <td><label for="c_email_provider_config">')
            # SOURCE LINE 58
            __M_writer(escape(_("E-mail provider config")))
            __M_writer(u'</label>: </td>\n\t        <td><textarea name="email_provider_config" class="required"  id="c_email_provider_config" cols=\'35\' rows=\'6\' maxlength="400">{}\t        \t\n\t        </textarea></td>\n        </tr>\n        <tr>\n\t        <td><label for="c_email_challenge_validity">')
            # SOURCE LINE 63
            __M_writer(escape(_("E-mail challenge validity (sec)")))
            __M_writer(u'</label>: </td>\n\t        <td><input type="text" name="email_challenge_validity" class="required"  id="c_email_challenge_validity" size="5" maxlength="5"></td>\n        </tr>\n        <tr>\n\t        <td><label for="c_email_blocking">')
            # SOURCE LINE 67
            __M_writer(escape(_("Time between e-mails (sec)")))
            __M_writer(u'</label>: </td>\n\t        <td><input type="text" name="email_blocking" class="required"  id="c_email_blocking" size="5" maxlength="5" value"30"></td>\n\t    </tr>\n    </table>\n</fieldset>\n</form>\n\n')
        # SOURCE LINE 75
        __M_writer(u'\n')
        # SOURCE LINE 76
        if c.scope == 'enroll.title' :
            # SOURCE LINE 77
            __M_writer(escape(_("E-mail token")))
            __M_writer(u'\n')
        # SOURCE LINE 79
        __M_writer(u'\n')
        # SOURCE LINE 80
        if c.scope == 'enroll' :
            # SOURCE LINE 81
            __M_writer(u'<script>\nfunction email_enroll_setup_defaults(config){\n\t// in case we enroll e-mail otp, we get the e-mail address of the user\n\temail_addresses = get_selected_email();\n\t$(\'#email_address\').val($.trim(email_addresses[0]));\n}\n\n/*\n * \'typ\'_get_enroll_params()\n *\n * this method is called, when the token  is submitted\n * - it will return a hash of parameters for admin/init call\n *\n */\nfunction email_get_enroll_params(){\n    var params = {};\n    // phone number\n    params[\'email_address\']\t= $(\'#email_address\').val();\n    params[\'description\'] = $(\'#email_address\').val() + " " + $(\'#enroll_email_desc\').val();\n    jQuery.extend(params, add_user_data());\n    return params;\n}\n</script>\n\n<table><tr>\n\t<td><label for="email_address">')
            # SOURCE LINE 106
            __M_writer(escape(_("E-mail address")))
            __M_writer(u'</label></td>\n\t<td><input type="text" name="email_address" id="email_address" value="" class="text ui-widget-content ui-corner-all"></td>\n</tr><tr>\n    <td><label for="enroll_email_desc" id=\'enroll_email_desc_label\'>')
            # SOURCE LINE 109
            __M_writer(escape(_("Description")))
            __M_writer(u'</label></td>\n    <td><input type="text" name="enroll_email_desc" id="enroll_email_desc" value="webGUI_generated" class="text" /></td>\n</tr>\n</table>\n\n')
        # SOURCE LINE 115
        __M_writer(u'\n')
        return ''
    finally:
        context.caller_stack._pop_frame()





#!/usr/bin/python
# -*- coding: ascii -*-
###########################################################################
# pbkdf2 - PKCS#5 v2.0 Password-Based Key Derivation
#
# Copyright (C) 2007-2011 Dwayne C. Litzenberger <dlitz@dlitz.net>
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#
# Country of origin: Canada
#
###########################################################################
# Sample PBKDF2 usage:
#   from Crypto.Cipher import AES
#   from pbkdf2 import PBKDF2
#   import os
#
#   salt = os.urandom(8)    # 64-bit salt
#   key = PBKDF2("This passphrase is a secret.", salt).read(32) # 256-bit key
#   iv = os.urandom(16)     # 128-bit IV
#   cipher = AES.new(key, AES.MODE_CBC, iv)
#     ...
#
# Sample crypt() usage:
#   from pbkdf2 import crypt
#   pwhash = crypt("secret")
#   alleged_pw = raw_input("Enter password: ")
#   if pwhash == crypt(alleged_pw, pwhash):
#       print "Password good"
#   else:
#       print "Invalid password"
#
###########################################################################

__version__ = "1.3"
__all__ = ['PBKDF2', 'crypt']

from struct import pack
import privacyidea.lib.crypto
import string
import sys

try:
    # Use PyCrypto (if available).
    from Crypto.Hash import HMAC, SHA as SHA1
except ImportError:
    # PyCrypto not available.  Use the Python standard library.
    import hmac as HMAC
    try:
        from hashlib import sha1 as SHA1
    except ImportError:
        # hashlib not available.  Use the old sha module.
        import sha as SHA1

#
# Python 2.1 thru 3.2 compatibility
#

if sys.version_info[0] == 2:
    _0xffffffffL = long(1) << 32
    def isunicode(s):
        return isinstance(s, unicode)
    def isbytes(s):
        return isinstance(s, str)
    def isinteger(n):
        return isinstance(n, (int, long))
    def b(s):
        return s
    def binxor(a, b):
        return "".join([chr(ord(x) ^ ord(y)) for (x, y) in zip(a, b)])
    def b64encode(data, chars="+/"):
        tt = string.maketrans("+/", chars)
        return data.encode('base64').replace("\n", "").translate(tt)
    from binascii import b2a_hex
else:
    _0xffffffffL = 0xffffffff
    def isunicode(s):
        return isinstance(s, str)
    def isbytes(s):
        return isinstance(s, bytes)
    def isinteger(n):
        return isinstance(n, int)
    def callable(obj):
        return hasattr(obj, '__call__')
    def b(s):
       return s.encode("latin-1")
    def binxor(a, b):
        return bytes([x ^ y for (x, y) in zip(a, b)])
    from base64 import b64encode as _b64encode
    def b64encode(data, chars="+/"):
        if isunicode(chars):
            return _b64encode(data, chars.encode('utf-8')).decode('utf-8')
        else:
            return _b64encode(data, chars)
    from binascii import b2a_hex as _b2a_hex
    def b2a_hex(s):
        return _b2a_hex(s).decode('us-ascii')
    xrange = range

class PBKDF2(object):
    """PBKDF2.py : PKCS#5 v2.0 Password-Based Key Derivation

    This implementation takes a passphrase and a salt (and optionally an
    iteration count, a digest module, and a MAC module) and provides a
    file-like object from which an arbitrarily-sized key can be read.

    If the passphrase and/or salt are unicode objects, they are encoded as
    UTF-8 before they are processed.

    The idea behind PBKDF2 is to derive a cryptographic key from a
    passphrase and a salt.

    PBKDF2 may also be used as a strong salted password hash.  The
    'crypt' function is provided for that purpose.

    Remember: Keys generated using PBKDF2 are only as strong as the
    passphrases they are derived from.
    """

    def __init__(self, passphrase, salt, iterations=1000,
                 digestmodule=SHA1, macmodule=HMAC):
        self.__macmodule = macmodule
        self.__digestmodule = digestmodule
        self._setup(passphrase, salt, iterations, self._pseudorandom)

    def _pseudorandom(self, key, msg):
        """Pseudorandom function.  e.g. HMAC-SHA1"""
        return self.__macmodule.new(key=key, msg=msg,
            digestmod=self.__digestmodule).digest()

    def read(self, bytes):
        """Read the specified number of key bytes."""
        if self.closed:
            raise ValueError("file-like object is closed")

        size = len(self.__buf)
        blocks = [self.__buf]
        i = self.__blockNum
        while size < bytes:
            i += 1
            if i > _0xffffffffL or i < 1:
                # We could return "" here, but
                raise OverflowError("derived key too long")
            block = self.__f(i)
            blocks.append(block)
            size += len(block)
        buf = b("").join(blocks)
        retval = buf[:bytes]
        self.__buf = buf[bytes:]
        self.__blockNum = i
        return retval

    def __f(self, i):
        # i must fit within 32 bits
        assert 1 <= i <= _0xffffffffL
        U = self.__prf(self.__passphrase, self.__salt + pack("!L", i))
        result = U
        for j in xrange(2, 1 + self.__iterations):
            U = self.__prf(self.__passphrase, U)
            result = binxor(result, U)
        return result

    def hexread(self, octets):
        """Read the specified number of octets. Return them as hexadecimal.

        Note that len(obj.hexread(n)) == 2*n.
        """
        return b2a_hex(self.read(octets))

    def _setup(self, passphrase, salt, iterations, prf):
        # Sanity checks:

        # passphrase and salt must be str or unicode (in the latter
        # case, we convert to UTF-8)
        if isunicode(passphrase):
            passphrase = passphrase.encode("UTF-8")
        elif not isbytes(passphrase):
            raise TypeError("passphrase must be str or unicode")
        if isunicode(salt):
            salt = salt.encode("UTF-8")
        elif not isbytes(salt):
            raise TypeError("salt must be str or unicode")

        # iterations must be an integer >= 1
        if not isinteger(iterations):
            raise TypeError("iterations must be an integer")
        if iterations < 1:
            raise ValueError("iterations must be at least 1")

        # prf must be callable
        if not callable(prf):
            raise TypeError("prf must be callable")

        self.__passphrase = passphrase
        self.__salt = salt
        self.__iterations = iterations
        self.__prf = prf
        self.__blockNum = 0
        self.__buf = b("")
        self.closed = False

    def close(self):
        """Close the stream."""
        if not self.closed:
            del self.__passphrase
            del self.__salt
            del self.__iterations
            del self.__prf
            del self.__blockNum
            del self.__buf
            self.closed = True

def crypt(word, salt=None, iterations=None):
    """PBKDF2-based unix crypt(3) replacement.

    The number of iterations specified in the salt overrides the 'iterations'
    parameter.

    The effective hash length is 192 bits.
    """

    # Generate a (pseudo-)random salt if the user hasn't provided one.
    if salt is None:
        salt = _makesalt()

    # salt must be a string or the us-ascii subset of unicode
    if isunicode(salt):
        salt = salt.encode('us-ascii').decode('us-ascii')
    elif isbytes(salt):
        salt = salt.decode('us-ascii')
    else:
        raise TypeError("salt must be a string")

    # word must be a string or unicode (in the latter case, we convert to UTF-8)
    if isunicode(word):
        word = word.encode("UTF-8")
    elif not isbytes(word):
        raise TypeError("word must be a string or unicode")

    # Try to extract the real salt and iteration count from the salt
    if salt.startswith("$p5k2$"):
        (iterations, salt, dummy) = salt.split("$")[2:5]
        if iterations == "":
            iterations = 400
        else:
            converted = int(iterations, 16)
            if iterations != "%x" % converted:  # lowercase hex, minimum digits
                raise ValueError("Invalid salt")
            iterations = converted
            if not (iterations >= 1):
                raise ValueError("Invalid salt")

    # Make sure the salt matches the allowed character set
    allowed = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789./"
    for ch in salt:
        if ch not in allowed:
            raise ValueError("Illegal character %r in salt" % (ch,))

    if iterations is None or iterations == 400:
        iterations = 400
        salt = "$p5k2$$" + salt
    else:
        salt = "$p5k2$%x$%s" % (iterations, salt)
    rawhash = PBKDF2(word, salt, iterations).read(24)
    return salt + "$" + b64encode(rawhash, "./")

# Add crypt as a static method of the PBKDF2 class
# This makes it easier to do "from PBKDF2 import PBKDF2" and still use
# crypt.
PBKDF2.crypt = staticmethod(crypt)

def _makesalt():
    """Return a 48-bit pseudorandom salt for crypt().

    This function is not suitable for generating cryptographic secrets.
    """
    binarysalt = b("").join([pack("@H", privacyidea.lib.crypto.urandom.randint(0, 0xffff)) for i in range(3)])
    return b64encode(binarysalt, "./")

# vim:set ts=4 sw=4 sts=4 expandtab:



# -*- coding: utf-8 -*-
#
#  privacyIDEA is a fork of LinOTP
#  May 08, 2014 Cornelius Kölbel
#  License:  AGPLv3
#  contact:  http://www.privacyidea.org
#
#  2014-09-22 Cornelius Kölbel, cornelius@privacyidea.org
#             some PEP8 things
#
#  Copyright (C) 2010 - 2014 LSE Leading Security Experts GmbH
#  License:  AGPLv3
#  contact:  http://www.linotp.org
#            http://www.lsexperts.de
#            linotp@lsexperts.de
#
# This code is free software; you can redistribute it and/or
# modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
# License as published by the Free Software Foundation; either
# version 3 of the License, or any later version.
#
# This code is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU AFFERO GENERAL PUBLIC LICENSE for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
'''The base Controller API

Provides the BaseController class for subclassing.
'''
import os

from pylons.i18n.translation import set_lang
from pylons.i18n import LanguageError

from pylons.controllers import WSGIController

from pylons import tmpl_context as c
from pylons import config
from pylons import request
from pylons import response

from privacyidea.lib.config import init_privacyIDEA_config
from privacyidea.lib.resolver import initResolvers
from privacyidea.lib.resolver import setupResolvers
from privacyidea.lib.resolver import closeResolvers

from privacyidea.model import meta
from privacyidea.lib.openid import SQLStorage
from privacyidea.model.meta import Session
from privacyidea import model

from privacyidea.lib.selftest import isSelfTest
from pylons.controllers.util import abort
from privacyidea.lib.util import check_session
from privacyidea.lib.account import is_admin_identity
from privacyidea.lib.user import User
from privacyidea.lib.util import getParam
from privacyidea.lib.log import log_with
from privacyidea.lib.audit import getAudit

ENCODING = "utf-8"

import logging
log = logging.getLogger(__name__)


@log_with(log)
def set_config(key, value, typ, description=None):
    '''
    create an intial config entry, if it does not exist

    :param key: the key
    :param value: the value
    :param description: the description of the key

    :return: nothing
    '''

    count = Session.query(model.Config).filter(model.Config.Key ==
                                               "privacyidea." + key).count()

    if count == 0:
        config_entry = model.Config(key,
                                    value,
                                    Type=typ,
                                    Description=description)
        Session.add(config_entry)

    return


@log_with(log)
def set_defaults():
    '''
    add privacyidea default config settings

    :return: - nothing -
    '''
    set_config(key=u"DefaultMaxFailCount",
               value=u"10", typ=u"int",
               description=u"The default maximum count for "
               "unsuccessful logins")

    set_config(key=u"DefaultCountWindow",
               value=u"10", typ=u"int",
               description=u"The default lookup window for "
               "tokens out of sync ")

    set_config(key=u"DefaultSyncWindow",
               value=u"1000", typ=u"int",
               description=u"The default lookup window for "
               "tokens out of sync ")

    set_config(key=u"DefaultChallengeValidityTime",
               value=u"120", typ=u"int",
               description=u"The default time, a challenge is "
               "regarded as valid.")

    set_config(key=u"DefaultResetFailCount",
               value=u"True", typ=u"bool",
               description=u"The default maximum count for unsucessful logins")

    set_config(key=u"DefaultOtpLen",
               value=u"6", typ=u"int",
               description=u"The default len of the otp values")

    set_config(key=u"PrependPin",
               value=u"True", typ=u"bool",
               description=u"is the pin prepended - most cases")

    set_config(key=u"FailCounterIncOnFalsePin",
               value=u"True", typ=u"bool",
               description=u"increment the FailCounter, if pin did not match")

    set_config(key=u"SMSProvider",
               value=u"smsprovider.HttpSMSProvider.HttpSMSProvider",
               typ=u"text",
               description=u"SMS Default Provider via HTTP")

    set_config(key=u"SMSProviderTimeout",
               value=u"300", typ=u"int",
               description=u"Timeout until registration must be done")

    set_config(key=u"SMSBlockingTimeout",
               value=u"30", typ=u"int",
               description=u"Delay until next challenge is created")

    set_config(key=u"DefaultBlockingTimeout",
               value=u"0", typ=u"int",
               description=u"Delay until next challenge is created")

    # setup for totp defaults
    # "privacyidea.totp.timeStep";"60";"None";"None"
    # "privacyidea.totp.timeWindow";"600";"None";"None"
    # "privacyidea.totp.timeShift";"240";"None";"None"

    set_config(key=u"totp.timeStep",
               value=u"30", typ=u"int",
               description=u"Time stepping of the time based otp token ")

    set_config(key=u"totp.timeWindow",
               value=u"300", typ=u"int",
               description=u"Lookahead time window of the time based "
               "otp token ")

    set_config(key=u"totp.timeShift",
               value=u"0", typ=u"int",
               description=u"Shift between server and totp token")

    set_config(key=u"AutoResyncTimeout",
               value=u"240", typ=u"int",
               description=u"Autosync timeout for an totp token")

    # setup for ocra defaults
    # OcraDefaultSuite
    # QrOcraDefaultSuite
    # OcraMaxChallenges
    # OcraChallengeTimeout

    set_config(key=u"OcraDefaultSuite",
               value=u"OCRA-1:HOTP-SHA256-8:C-QN08", typ=u"string",
               description=u"Default OCRA suite for an ocra token ")

    set_config(key=u"QrOcraDefaultSuite",
               value=u"OCRA-1:HOTP-SHA256-8:C-QA64", typ=u"int",
               description=u"Default OCRA suite for an ocra qr token ")

    set_config(key=u"OcraMaxChallenges",
               value=u"4", typ=u"int",
               description=u"Maximum open ocra challenges")

    set_config(key=u"OcraChallengeTimeout",
               value=u"300", typ=u"int",
               description=u"Timeout for an open ocra challenge")

    # emailtoken defaults
    set_config(key=u"EmailProvider",
               value="privacyidea.lib.emailprovider.SMTPEmailProvider",
               typ=u"string",
               description=u"Default EmailProvider class")
    set_config(key=u"EmailChallengeValidityTime",
               value="600", typ=u"int",
               description=u"Time that an e-mail token challenge stays "
               "valid (seconds)")
    set_config(key=u"EmailBlockingTimeout",
               value="120", typ=u"int",
               description=u"Time during which no new e-mail is sent out")


@log_with(log)
def setup_app(conf, conf_global=None, unitTest=False):
    '''
    setup_app is the hook, which is called, when the application is created

    :param conf: the application configuration

    :return: - nothing -
    '''
    if conf_global is not None:
        if "sqlalchemy.url" in conf_global:
            log.info("sqlalchemy.url")
    else:
        conf.get("sqlalchemy.url", None)

    if unitTest is True:
        log.info("Deleting previous tables...")
        meta.metadata.drop_all(bind=meta.engine)

    # Create the tables if they don't already exist
    log.info("Creating tables...")
    meta.metadata.create_all(bind=meta.engine)

    if "privacyideaSecretFile" in conf:
        filename = conf.get("privacyideaSecretFile")
        try:
            with open(filename):
                pass
        except IOError:
            log.warning("The privacyIDEA Secret File could not be found " +
                        "-creating a new one: %s" % filename)
            f_handle = open(filename, 'ab+')
            secret = os.urandom(32 * 5)
            f_handle.write(secret)
            f_handle.close()
            os.chmod(filename, 0400)
        log.info("privacyideaSecretFile: %s" % filename)

    set_defaults()
    Session.commit()
    log.info("Successfully set up.")


class BaseController(WSGIController):
    """
    BaseController class - will be called with every request
    """

    def __init__(self, *args, **kw):
        """
        base controller constructor

        :param *args: generic argument array
        :param **kw: generic argument dict
        :return: None

        """
        self.parent = super(WSGIController, self)
        self.parent.__init__(*args, **kw)
        self.audit = getAudit()
        self.audit.initialize()

        # make the OpenID SQL Instance globally available
        openid_sql = config.get('openid_sql', None)
        if openid_sql is None:
            try:
                openid_storage = SQLStorage()
                config['openid_sql'] = openid_storage
            except Exception as exx:
                config['openid_sql'] = exx
                log.error("Failed to configure openid_sql: %r" % exx)

        app_setup_done = config.get('app_setup_done', False)
        if app_setup_done is False:
            try:
                setup_app(config)
                config['app_setup_done'] = True
            except Exception as exx:
                config['app_setup_done'] = False
                log.error("Failed to serve request: %r" % exx)
                raise exx

        l_config = init_privacyIDEA_config()

        resolver_setup_done = config.get('resolver_setup_done', False)
        if resolver_setup_done is False:
            try:
                cache_dir = config.get("app_conf", {}).get("cache_dir", None)
                setupResolvers(config=l_config, cache_dir=cache_dir)
                config['resolver_setup_done'] = True
            except Exception as exx:
                config['resolver_setup_done'] = False
                log.error("Failed to setup resolver: %r" % exx)
                raise exx

        initResolvers()
        
        # Add protection against click jacking
        response.headers["X-Frame-Options"] = "SAMEORIGIN"

        c.help_url = config.get('help_url')

        return

    def __call__(self, environ, start_response):
        '''Invoke the Controller'''
        # WSGIController.__call__ dispatches to the Controller method
        # the request is routed to. This routing information is
        # available in environ['pylons.routes_dict']

        from privacyidea.lib.config import getGlobalObject

        glo = getGlobalObject()
        sep = glo.security_provider

        try:
            hsm = sep.getSecurityModule()
            c.hsm = hsm
            ret = WSGIController.__call__(self, environ, start_response)
        finally:
            meta.Session.remove()
            # free the lock on the scurityPovider if any
            sep.dropSecurityModule()
            closeResolvers()

        return ret

    @log_with(log)
    def before_identity_check(self, action="", check_admin=True):
        '''
        This is a function that can be called in the __before__ method
        to check the identity
        '''
        param = request.params
        if isSelfTest():
            log.debug("Doing selftest!")
            uuser = getParam(param, "selftest_user", True)
            if uuser is not None:
                (c.user, _foo, c.realm) = uuser.rpartition('@')
            else:
                c.realm = ""
                c.user = "--u--"
                env = request.environ
                uuser = env.get('REMOTE_USER')
                if uuser is not None:
                    (c.user, _foo, c.realm) = uuser.rpartition('@')

            self.authUser = User(c.user, c.realm, '')
            log.debug("authenticating as %s in realm %s!" % (c.user, c.realm))
        else:
            identity = request.environ.get('repoze.who.identity')
            if identity is None:
                abort(401, "You are not authenticated")
                
            # checking the session
            if (False == check_session(request)):
                c.audit['action'] = request.path[1:]
                c.audit['info'] = "session expired"
                self.audit.log(c.audit)
                abort(401, "No valid session")
                
            log.debug("doing getAuthFromIdentity in action %s" % action)

            user_id = request.environ.get('repoze.who.identity').get('repoze.who.userid')
            if type(user_id) == unicode:
                user_id = user_id.encode(ENCODING)
            identity = user_id.decode(ENCODING)
            log.debug("getting identity from repoze.who: %r" % identity)

            (c.user, _foo, c.realm) = identity.rpartition('@')
            if check_admin:
                is_admin_identity(identity)
            self.authUser = User(c.user, c.realm, '')

            log.debug("set self.authUser to: %s, %s" % (self.authUser.login,
                                                        self.authUser.realm))
            log.debug('param for action %s: %s' % (action, param))


            
    @log_with(log)
    def set_language(self):
        '''Invoke before everything else. And set the translation language'''
        languages = request.headers.get('Accept-Language', '').split(';')
        found_lang = False

        for language in languages:
            for lang in language.split(','):
                try:
                    if lang == "en" or lang == "en-gb" or lang == "en-GB":
                        found_lang = True
                        break
                    set_lang(lang)
                    found_lang = True
                    break
                except LanguageError as _exx:
                    pass

            if found_lang is True:
                break

        if found_lang is False:
            log.warning("Cannot set preferred language: %r" % languages)

        return

    def __enter__(self):
        pass
    
    def __exit__(self, typ, value, traceback):
        pass



# -*- coding: utf-8 -*-
#
#  privacyIDEA
#  May 11, 2014 Cornelius Kölbel, info@privacyidea.org
#  http://www.privacyidea.org
#
# This code is free software; you can redistribute it and/or
# modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
# License as published by the Free Software Foundation; either
# version 3 of the License, or any later version.
#
# This code is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU AFFERO GENERAL PUBLIC LICENSE for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#
import logging
log = logging.getLogger(__name__)
from privacyidea.lib.audit import AuditBase


from sqlalchemy import Table, MetaData, Column
from sqlalchemy import Integer, String, DateTime, asc, desc, and_
from sqlalchemy.orm import mapper
import datetime
# from privacyidea.lib.config  import get_privacyIDEA_config
from pylons import config as ini_config
import traceback
from Crypto.Hash import SHA256 as HashFunc
from Crypto.PublicKey import RSA
from sqlalchemy.exc import OperationalError
from privacyidea.lib.log import log_with

metadata = MetaData()

logentry = Table('pidea_audit',
                 metadata,
                 Column('id', Integer, primary_key=True),
                 Column('date', DateTime),
                 Column('signature', String(620)),
                 Column('action', String(50)),
                 Column('success', Integer),
                 Column('serial', String(20)),
                 Column('token_type', String(12)),
                 Column('user', String(20)),
                 Column('realm', String(20)),
                 Column('administrator', String(20)),
                 Column('action_detail', String(50)),
                 Column('info', String(50)),
                 Column('privacyidea_server', String(20)),
                 Column('client', String(20)),
                 Column('loglevel', String(12)),
                 Column('clearance_level', String(12))
                 )


class LogEntry(object):
    def __init__(self,
                 action="",
                 success=0,
                 serial="",
                 token_type="",
                 user="",
                 realm="",
                 administrator="",
                 action_detail="",
                 info="",
                 privacyidea_server="",
                 client="",
                 loglevel="default",
                 clearance_level="default"
                 ):
        self.signature = ""
        self.date = datetime.datetime.now()
        self.action = action
        self.success = success
        self.serial = serial
        self.token_type = token_type
        self.user = user
        self.realm = realm
        self.administrator = administrator
        self.action_detail = action_detail
        self.info = info
        self.privacyidea_server = privacyidea_server
        self.client = client
        self.loglevel = loglevel
        self.clearance_level = clearance_level

mapper(LogEntry, logentry)

from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker


class Audit(AuditBase):
    '''
    This is the SQLAudit module, which writes the audit entries
    to an SQL database table.
    It requires the configuration parameters:
    
    privacyideaAudit.sql.url (default: sqlalchemy.url)
    
    '''
    
    def __init__(self):
        self.name = "sqlaudit"
        self.readKeys()
        
        # conf = get_privacyIDEA_config()
        # an Engine, which the Session will use for connection
        # resources
        connect_string = ini_config.get("privacyideaAudit.sql.url",
                                        ini_config.get("sqlalchemy.url"))
        log.info("using the connect string %s" % connect_string)
        self.engine = create_engine(connect_string)

        # create a configured "Session" class
        Session = sessionmaker(bind=self.engine)

        # create a Session
        self.session = Session()
        try:
            metadata.create_all(self.engine)
        except OperationalError as exx:
            log.info("%r" % exx)
            pass
            
    def getAuditId(self):
        return self.name

    def _create_filter(self, param):
        '''
        create a filter condition for the logentry
        '''
        conditions = []
        for search_key in param.keys():
            search_value = param.get(search_key)
            if search_value.strip() != '':
                try:
                    conditions.append(getattr(LogEntry,
                                              search_key).like("%" +
                                                               search_value +
                                                               "%"))
                except:
                    # The search_key was no search key but some
                    # bullshit stuff in the param
                    pass
        # Combine them with or to a BooleanClauseList
        filter_condition = and_(*conditions)
        return filter_condition

    def getTotal(self, param, AND=True, display_error=True):
        '''
        This method returns the total number of audit entries
        in the audit store
        '''
        count = 0

        # if param contains search filters, we build the search filter
        # to only return the number of those entries
        filter_condition = self._create_filter(param)
        
        try:
            count = self.session.query(LogEntry.id)\
                .filter(filter_condition)\
                .count()
        finally:
            self.session.close()
        return count

    @log_with(log)
    def log(self, param):
        '''
        This method is used to log the data.
        It should hash the data and do a hash chain and sign the data
        '''
        try:
            le = LogEntry(action=param.get("action"),
                          success=param.get("success"),
                          serial=param.get("serial"),
                          token_type=param.get("token_type"),
                          user=param.get("user"),
                          realm=param.get("realm"),
                          administrator=param.get("administrator"),
                          action_detail=param.get("action_detail"),
                          info=param.get("info"),
                          privacyidea_server=param.get("privacyidea_server"),
                          client=param.get("client", ""),
                          loglevel=param.get("log_level"),
                          clearance_level=param.get("clearance_level")
                          )
            self.session.add(le)
            self.session.commit()
            # Add the signature
            s = self._log_to_string(le)
            sign = self._sign(s)
            le.signature = sign
            self.session.merge(le)
            self.session.commit()
        except Exception as exx:
            log.error("exception %r" % exx)
            log.error("%s" % traceback.format_exc())
            self.session.rollback()

        finally:
            self.session.close()
    
    def _sign(self, s):
        '''
        Create a signature of the string s
        
        :return: The signature of the string
        :rtype: long
        '''
        RSAkey = RSA.importKey(self.private)
        hashvalue = HashFunc.new(s).digest()
        signature = RSAkey.sign(hashvalue, 1)
        s_signature = str(signature[0])
        return s_signature
    
    def _verify_sig(self, audit_entry):
        '''
        Check the signature of the audit log in the database
        '''
        r = False
        try:
            RSAkey = RSA.importKey(self.public)
            hashvalue = HashFunc.new(self._log_to_string(audit_entry)).digest()
            signature = long(audit_entry.signature)
            r = RSAkey.verify(hashvalue, (signature,))
        except Exception:
            log.error("Failed to verify audit entry: %r" % audit_entry.id)
            log.error(traceback.format_exc())
        return r

    def _check_missing(self, audit_id):
        '''
        Check if the audit log contains the entries before and after
        the given id.
        
        TODO: We can not check at the moment if the first or the last entries
        were deleted. If we want to do this, we need to store some signed
        meta information
        1. Which one was the first entry. (use initialize_log)
        2. Which one was the last entry.
        '''
        res = False
        try:
            id_bef = self.session.query(LogEntry.id
                                        ).filter(LogEntry.id ==
                                                 int(audit_id) - 1).count()
            id_aft = self.session.query(LogEntry.id
                                        ).filter(LogEntry.id ==
                                                 int(audit_id) + 1).count()
            # We may not do a commit!
            # self.session.commit()
            if id_bef and id_aft:
                res = True
        except Exception as exx:
            log.error("exception %r" % exx)
            log.error("%s" % traceback.format_exc())
            # self.session.rollback()
        finally:
            # self.session.close()
            pass
            
        return res
    
    def _log_to_string(self, le):
        '''
        This function creates a string from the logentry so
        that this string can be signed.
        
        Note: Not all elements of the LogEntry are used to generate the
        string (the Signature is not!), otherwise we could have used pickle
        '''
        s = "id=%s,date=%s,action=%s,succ=%s,serial=%s,t=%s,u=%s,r=%s,adm=%s,"\
            "ad=%s,i=%s,ps=%s,c=%s,l=%s,cl=%s" % (le.id,
                                                  le.date,
                                                  le.action,
                                                  le.success,
                                                  le.serial,
                                                  le.token_type,
                                                  le.user,
                                                  le.realm,
                                                  le.administrator,
                                                  le.action_detail,
                                                  le.info,
                                                  le.privacyidea_server,
                                                  le.client,
                                                  le.loglevel,
                                                  le.clearance_level)
        return s
        
    def _get_logentry_attribute(self, key):
        '''
        This function returns the LogEntry attribute for the given key value
        '''
        sortname = {'number': LogEntry.id,
                    'action': LogEntry.action,
                    'success': LogEntry.success,
                    'serial': LogEntry.serial,
                    'date': LogEntry.date,
                    'token_type': LogEntry.token_type,
                    'user': LogEntry.user,
                    'realm': LogEntry.realm,
                    'administrator': LogEntry.administrator,
                    'action_detail': LogEntry.action_detail,
                    'info': LogEntry.info,
                    'privacyidea_server': LogEntry.privacyidea_server,
                    'client': LogEntry.client,
                    'loglevel': LogEntry.loglevel,
                    'clearance_level': LogEntry.clearance_level}
        return sortname.get(key)
        
    def search(self, search_dict, rp_dict):
        '''
        This function returns the audit log as a list of dictionaries.
        '''
        res = []
        auditIter = self.searchQuery(search_dict, rp_dict=rp_dict)
        try:
            le = auditIter.next()
            while le:
                # Fill the list
                res.append(self.audit_entry_to_dict(le))
                le = auditIter.next()
        except StopIteration:
            pass
        return res
        
    def searchQuery(self, search_dict, rp_dict):
        '''
        This function returns the audit log as an iterator on the result
        '''
        logentries = None
        try:
            limit = int(rp_dict.get('rp', 15))
            offset = (int(rp_dict.get('page', 1)) - 1) * limit
            
            # create filter condition
            filter_condition = self._create_filter(search_dict)
            
            if rp_dict.get("sortorder") == "desc":
                logentries = self.session.query(LogEntry)\
                                         .filter(filter_condition)\
                                         .order_by(desc(self._get_logentry_attribute(rp_dict.get("sortname"))))\
                                         .limit(limit)\
                                         .offset(offset)
            else:
                logentries = self.session.query(LogEntry)\
                                         .filter(filter_condition)\
                                         .order_by(asc(self._get_logentry_attribute(rp_dict.get("sortname"))))\
                                         .limit(limit)\
                                         .offset(offset)
                                         
        except Exception as exx:
            log.error("exception %r" % exx)
            log.error("%s" % traceback.format_exc())
            self.session.rollback()
        finally:
            self.session.close()

        if logentries is None:
            return iter([])
        else:
            return iter(logentries)
    
    def audit_entry_to_dict(self, audit_entry):
        sig = self._verify_sig(audit_entry)
        is_not_missing = self._check_missing(int(audit_entry.id))
        # is_not_missing = True
        audit_dict = {'number': audit_entry.id,
                      'date': audit_entry.date.isoformat(),
                      'sig_check': "OK" if sig else "FAIL",
                      'missing_line': "OK" if is_not_missing else "FAIL",
                      'action': audit_entry.action,
                      'success': audit_entry.success,
                      'serial': audit_entry.serial,
                      'token_type': audit_entry.token_type,
                      'user': audit_entry.user,
                      'realm': audit_entry.realm,
                      'administrator': audit_entry.administrator,
                      'action_detail': audit_entry.action_detail,
                      'info': audit_entry.info,
                      'privacyidea_server': audit_entry.privacyidea_server,
                      'client': audit_entry.client,
                      'log_level': audit_entry.loglevel,
                      'clearance_level': audit_entry.clearance_level
                      }
        return audit_dict
    
#########################################################################
#
# To be run at the command line to clean the logs
#

import os
import sys
from getopt import getopt, GetoptError
import ConfigParser


def usage():
    print('''cleanup audit database according to:
        privacyideaAudit.sql.highwatermark and
        privacyideaAudit.sql.lowwatermark
        
    Parameter:
    -f, --file <privacyidea.ini file>
    --high <high watermark>
    --low  <low wartermark>
    ''')


def cleanup_db(filename, highwatermark=None, lowwatermark=None):

    config_path = os.path.abspath(os.path.dirname(filename))
    config = ConfigParser.ConfigParser()
    config.read(filename)
    # Set the current path, which we might need, if we are using sqlite
    config.set("DEFAULT", "here", config_path)
    config.set("app:main", "here", config_path)
    # get the type - we only work for sqlaudit
    audit_type = config.get("DEFAULT", "privacyideaAudit.type")
    if audit_type != "privacyidea.lib.auditmodules.sqlaudit":
        raise Exception("We only work with audit type sql. %s given."
                        % audit_type)
    
    if not highwatermark:
        try:
            highwatermark = config.get("DEFAULT",
                                       "privacyideaAudit.sql.highwatermark")
        except ConfigParser.NoOptionError:
            highwatermark = 10000
        
    if not lowwatermark:
        try:
            lowwatermark = config.get("DEFAULT",
                                      "privacyideaAudit.sql.lowwatermark")
        except ConfigParser.NoOptionError:
            lowwatermark = 5000
    
    try:
        sql_url = config.get("DEFAULT", "privacyideaAudit.sql.url")
    except ConfigParser.NoOptionError:
        sql_url = config.get("app:main", "sqlalchemy.url")
        
    print "Cleaning up with high: %s, low: %s. %s" % (highwatermark,
                                                      lowwatermark,
                                                      sql_url)
    
    engine = create_engine(sql_url)
    # create a configured "Session" class
    session = sessionmaker(bind=engine)()
    # create a Session
    metadata.create_all(engine)
    count = session.query(LogEntry.id).count()
    for l in session.query(LogEntry.id).order_by(desc(LogEntry.id)).limit(1):
        last_id = l[0]
    print "The log audit log has %i entries, the last one is %i" % (count,
                                                                    last_id)
    # deleting old entries
    if count > highwatermark:
        print "More than %i entries, deleting..." % highwatermark
        cut_id = last_id - lowwatermark
        # delete all entries less than cut_id
        print "Deleting entries smaller than %i" % cut_id
        session.query(LogEntry.id).filter(LogEntry.id < cut_id).delete()
        session.commit()
    

def main():  # pragma: no cover

    filename = None
    highwatermark = None
    lowwatermark = None
    try:
        opts, _args = getopt(sys.argv[1:], "hf:", ["help",
                                                   "file=",
                                                   "high=",
                                                   "low="])

    except GetoptError:
        usage()
        sys.exit(1)

    for opt, arg in opts:
        if opt in ('-f', '--file'):
            filename = arg
        elif opt in ('-h', '--help'):
            usage()
            sys.exit(1)
        elif opt in ('--high'):
            highwatermark = int(arg)
        elif opt in ('--low'):
            lowwatermark = int(arg)

    if filename:
        cleanup_db(filename,
                   highwatermark=highwatermark,
                   lowwatermark=lowwatermark)
    else:
        usage()
        sys.exit(2)

if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
#
#
#  privacyIDEA is a fork of LinOTP
#  May 28, 2014 Cornelius Kölbel
#  License:  AGPLv3
#  contact:  http://www.privacyidea.org
#
#  Copyright (C) 2010 - 2014 LSE Leading Security Experts GmbH
#  License:  AGPLv3
#  contact:  http://www.linotp.org
#            http://www.lsexperts.de
#            linotp@lsexperts.de
#
#    This program is free software: you can redistribute it and/or
#    modify it under the terms of the GNU Affero General Public
#    License, version 3, as published by the Free Software Foundation.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the
#               GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#
"""
testing controller - for testing purposes only
This controller is only used in the test scripts.
"""

import logging

from pylons import (request,
                    response,
                    config,
                    tmpl_context as c)

from privacyidea.lib.config import get_privacyIDEA_config
from privacyidea.lib.base import BaseController
from privacyidea.lib.util import getParam
from privacyidea.lib.user import getUserFromParam

from privacyidea.lib.reply import sendResult, sendError

from privacyidea.model.meta import Session
from privacyidea.lib.selftest import isSelfTest
from privacyidea.lib.policy import PolicyClass

import traceback

optional = True
required = False

log = logging.getLogger(__name__)

# from paste.debug.profile import profile_decorator


class TestingController(BaseController):

    def __before__(self):
        return response

    def __after__(self):
        return response

    def autosms(self):
        '''
        This function is used to test the autosms policy

        method:
            testing/autosms

        arguments:
            user    - username / loginname
            realm   - additional realm to match the user to a useridresolver


        returns:
            JSON response
        '''
        log.debug('[autosms]')

        param = request.params
        try:

            if isSelfTest() is False:
                Session.rollback()
                return sendError(response,
                                 "The testing controller can "
                                 "only be used in SelfTest mode!", 0)

            _user = getUserFromParam(param, required)
            Policy = PolicyClass(request,
                                 config,
                                 c,
                                 get_privacyIDEA_config())
            ok = Policy.get_auth_AutoSMSPolicy()

            Session.commit()
            return sendResult(response, ok, 0)

        except Exception as e:
            log.error("[autosms] validate/check failed: %r", e)
            log.error("[autosms] %s" % traceback.format_exc())
            Session.rollback()
            return sendError(response,
                             "validate/check failed:" + unicode(e), 0)

        finally:
            Session.close()
            log.debug('[autosms] done')

    def http2sms(self):
        '''
        This function simulates an HTTP SMS Gateway.

        method:
            test/http2sms

        arguments:

           * sender, absender
           * from, von
           * destination, ziel
           * password, passwort
           * from, von
           * text
           * account
           * api_id


        returns:
           As this is a test controller, the response depends on the input
           values.

            account = 5vor12, sender = legit
                -> Response Success: "200" (Text)

            account = 5vor12, sender = <!legit>
                -> Response Failed: "Failed" (Text)

            account = clickatel, username = legit
                -> Response Success: "ID <Random Number>" (Text)

            account = clickatel, username = <!legit>
                -> Response Success: "FAILED" (Text)
        '''
        log.debug('[http2sms]')
        param = request.params

        try:
            account = getParam(param, "account", optional=False)
            sender = getParam(param, "sender", optional=True)
            username = getParam(param, "username", optional=True)

            destination = getParam(param, "destination")
            if not destination:
                destination = getParam(param, "ziel")

            text = getParam(param, "text")
            if not text:
                text = getParam(param, "text")

            if not destination:
                raise Exception("Missing <destination>")

            if not text:
                raise Exception("Missing <text>")

            if account == "5vor12":
                if sender == "legit":
                    return "200"
                else:
                    return "Failed"

            if account == "clickatel":
                if username == "legit":
                    return "ID 1000"
                else:
                    return "FAILED"

            Session.commit()
            return "Missing account info."

        except Exception as e:
            log.error('[http2sms] %r' % e)
            log.error("[http2sms] %s" % traceback.format_exc())
            Session.rollback()
            return sendError(response, unicode(e), 0)

        finally:
            Session.close()
            log.debug('[http2sms] done')

# eof######################################################################



# -*- coding: utf-8 -*-
#
#  privacyIDEA is a fork of LinOTP
#  May 08, 2014 Cornelius Kölbel
#  License:  AGPLv3
#  contact:  http://www.privacyidea.org
#
#  Copyright (C) 2010 - 2014 LSE Leading Security Experts GmbH
#  License:  LSE
#  contact:  http://www.linotp.org
#            http://www.lsexperts.de
#            linotp@lsexperts.de

'''
  Description:  This file contains the definition of the password token class
  
  Dependencies: -

'''

import logging
from privacyidea.lib.crypto   import zerome

from privacyidea.lib.log import log_with

optional = True
required = False

from privacyidea.lib.tokenclass import TokenClass

log = logging.getLogger(__name__)

###############################################
class PasswordTokenClass(TokenClass):
    '''
    This Token does use a fixed Password as the OTP value.
    In addition, the OTP PIN can be used with this token.
    This Token can be used for a scenario like losttoken
    '''

    class __secretPassword__(object):

        def __init__(self, secObj):
            self.secretObject = secObj

        def getPassword(self):
            return self.secretObject.getKey()

        def checkOtp(self, anOtpVal):
            res = -1

            key = self.secretObject.getKey()

            if key == anOtpVal:
                res = 0

            zerome(key)
            del key

            return res

    def __init__(self, aToken):
        TokenClass.__init__(self, aToken)
        self.hKeyRequired = True
        self.setType(u"pw")

    @classmethod
    def getClassType(cls):
        return "pw"

    @classmethod
    @log_with(log)
    def getClassInfo(cls, key=None, ret='all'):
        '''
        getClassInfo - returns a subtree of the token definition

        :param key: subsection identifier
        :type key: string

        :param ret: default return value, if nothing is found
        :type ret: user defined

        :return: subsection if key exists or user defined
        :rtype: s.o.

        '''
        res = {
               'type'           : 'pw',
               'title'          : 'Password Token',
               'description'    : ('A token with a fixed password. Can be combined with the OTP PIN. Is used for the lost token scenario.'),
               'init'         : {},
               'config'        : {},
               'selfservice'   :  {},
               'policy' : {},
               }
        # I don't think we need to define the lost token policies here...

        if key is not None and res.has_key(key):
            ret = res.get(key)
        else:
            if ret == 'all':
                ret = res

        return ret



    def update(self, param):

        TokenClass.update(self, param)
        # The otplen is determined by the otpkey. So we
        # call the setOtpLen after the parents update, to overwrite
        # specified OTP lengths with the length of the password
        self.setOtpLen(0)

    @log_with(log)
    def setOtpLen(self, otplen):
        '''
        sets the OTP length to the length of the password
        '''
        secretHOtp = self.token.getHOtpKey()
        sp = PasswordTokenClass.__secretPassword__(secretHOtp)
        pw_len = len(sp.getPassword())
        TokenClass.setOtpLen(self, pw_len)
        return

    @log_with(log, log_entry=False)
    def checkOtp(self, anOtpVal, counter, window, options=None):
        '''
        This checks the static password
        '''
        log.debug("checkOtp of PasswordToken")

        secretHOtp = self.token.getHOtpKey()
        sp = PasswordTokenClass.__secretPassword__(secretHOtp)
        res = sp.checkOtp(anOtpVal)

        return res



# -*- coding: utf-8 -*-
#
#  privacyIDEA is a fork of LinOTP
#  May 08, 2014 Cornelius Kölbel
#  License:  AGPLv3
#  contact:  http://www.privacyidea.org
#
#  Copyright (C) 2010 - 2014 LSE Leading Security Experts GmbH
#  License:  AGPLv3
#  contact:  http://www.linotp.org
#            http://www.lsexperts.de
#            linotp@lsexperts.de
#
# This code is free software; you can redistribute it and/or
# modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
# License as published by the Free Software Foundation; either
# version 3 of the License, or any later version.
#
# This code is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU AFFERO GENERAL PUBLIC LICENSE for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
'''
This file is part of the privacyidea service
'''
import json
import re
import webob

from privacyidea.lib.selftest import isSelfTest
from pylons import request, response, config, tmpl_context as c
from privacyidea.config.environment import get_resolver_list as getlist
from privacyidea.model.meta import Session

from privacyidea.lib.base import BaseController

from privacyidea.lib.config import (storeConfig,
                                    getFromConfig,
                                    updateConfig,
                                    removeFromConfig)

from privacyidea.lib.realm import (setDefaultRealm,
                                   isRealmDefined)

from privacyidea.weblib.util import get_client

from privacyidea.lib.resolver import (defineResolver,
                                      checkResolverType,
                                      getResolverList,
                                      getResolverInfo,
                                      deleteResolver,
                                      get_resolver_name)

from privacyidea.lib.error import ParameterError

from privacyidea.lib.util import (remove_session_from_param,
                                  getParam, getLowerParams)
from privacyidea.lib.reply import (sendResult, sendError, sendXMLResult,
                                   sendXMLError)
from privacyidea.lib.realm import (getRealms,
                                   getDefaultRealm)
from privacyidea.lib.user import (setRealm,
                                  getUserFromRequest)

from privacyidea.lib.realm import deleteRealm
from privacyidea.lib.token import (newToken,
                                   get_token_type_list,
                                   get_policy_definitions)
from privacyidea.lib.policy import PolicyClass
from privacyidea.lib.policy import PolicyException
from privacyidea.lib.config import get_privacyIDEA_config

from privacyidea.weblib.policy import setPolicy
from privacyidea.weblib.policy import deletePolicy
from privacyidea.lib.log import log_with
from paste.fileapp import FileApp
from cgi import escape
from pylons.i18n.translation import _

ENCODING = "utf-8"

import traceback
import logging
log = logging.getLogger(__name__)

optional = True
required = False


class SystemController(BaseController):

    '''
    The privacyidea.controllers are the implementation of the web-API to
    talk to the privacyIDEA server.
    The SystemController is used to configure the privacyIDEA server.
    The functions of the SystemController are invoked like this

        https://server/system/<functionname>

    The functions are described below in more detail.
    '''

    @log_with(log)
    def __before__(self, action, **params):
        '''
        __before__ is called before every action
             so we can check the authorization (fixed?)

        :param action: name of the to be called action
        :param params: the list of http parameters

        :return: return response
        :rtype:  pylon response
        '''
        try:
            c.audit['success'] = False
            c.audit['client'] = get_client()
            self.Policy = PolicyClass(request, config, c,
                                      get_privacyIDEA_config(),
                                      token_type_list=get_token_type_list())
            self.before_identity_check(action)
            
            # check authorization
            if action not in ["_add_dynamic_tokens", 'setupSecurityModule']:
                self.Policy.checkPolicyPre('system', action)

            # default return for the __before__ and __after__
            return response

        except PolicyException as pex:
            log.error("%r: policy exception %r" % (action, pex))
            log.error(traceback.format_exc())
            Session.rollback()
            Session.close()
            return sendError(response, pex, context='before')

        except webob.exc.HTTPUnauthorized as acc:
            # the exception, when an abort() is called if forwarded
            log.error("%r: webob.exception %r" % (action, acc))
            log.error(traceback.format_exc())
            Session.rollback()
            Session.close()
            raise acc

        except Exception as exx:  # pragma: no cover
            log.error("%r: exception %r" % (action, exx))
            log.error(traceback.format_exc())
            Session.rollback()
            Session.close()
            return sendError(response, exx, context='before')

        finally:
            pass

    @log_with(log)
    def __after__(self, action, **params):
        '''
        __after is called after every action

        :return: return the response
        :rtype:  pylons response
        '''
        try:
            c.audit['administrator'] = getUserFromRequest(request).get("login")
            self.audit.log(c.audit)
            # default return for the __before__ and __after__
            return response

        except Exception as exx:  # pragma: no cover
            log.error("exception %r" % (exx))
            log.error(traceback.format_exc())
            Session.rollback()
            Session.close()
            return sendError(response, exx, context='after')

        finally:
            pass


########################################################

    def setDefault(self):
        """
        method:
            system/set

        description:
            define default settings for tokens. These default settings
            are used when new tokens are generated. The default settings will
            not affect already enrolled tokens.

        arguments:
            DefaultMaxFailCount    - Default value for the maximum allowed
                                     authentication failures
            DefaultSyncWindow      - Default value for the synchronization
                                     window
            DefaultCountWindow     - Default value for the coutner window
            DefaultOtpLen          - Default value for the OTP value length --
                                     usually 6 or 8
            DefaultResetFailCount  - Default value, if the FailCounter should
                                     be reset on successful authentication
                                     [True|False]


        returns:
            a json result with a boolean
              "result": true

        exception:
            if an error occurs an exception is serialized and returned

        """
        res = {}
        count = 0
        description = "setDefault: parameters are\
        DefaultMaxFailCount\
        DefaultSyncWindow\
        DefaultCountWindow\
        DefaultOtpLen\
        DefaultResetFailCount\
        "

        keys = ["DefaultMaxFailCount",
                "DefaultSyncWindow",
                "DefaultCountWindow",
                "DefaultOtpLen",
                "DefaultResetFailCount"]

        # config settings from here
        try:
            param = getLowerParams(request.params)

            for k in keys:
                if k.lower() in param:
                    value = getParam(param, k.lower(), required)
                    ret = storeConfig(k, value)
                    des = "set " + k
                    res[des] = ret
                    count = count + 1

                    c.audit['success'] = count
                    c.audit['info'] += "%s=%s, " % (k, value)

            if count == 0:
                log.warning("Failed saving config. Could not find any "
                            "known parameter. %s"
                            % description)
                raise ParameterError("Usage: %s" % description, id=77)

            Session.commit()
            return sendResult(response, res)

        except Exception as exx:  # pragma: no cover
            log.error('commit failed: %r' % exx)
            log.error(traceback.format_exc())
            Session.rollback()
            return sendError(response, exx)

        finally:
            Session.close()


########################################################
    def setConfig(self):
        """
        set a configuration key or a set of configuration entries

        parameter could either be in the form key=..&value=..
        or as a set of generic keyname=value pairs.

        *remark: In case of key-value pairs the type information could be
                 provided by an additional parameter with same keyname with the
                 postfix ".type". Value could then be 'password' to trigger the
                 storing of the value in an encrypted form

        :param key: configuration entry name
        :param value: configuration value
        :param type: type of the value: int or string/text or password
                     password will trigger to store the encrypted value
        :param description: additional information for this config entry

        * or
        :param key-value pairs: pair of &keyname=value pairs
        :return: a json result with a boolean "result": true
        """

        res = {}
        param = {}

        try:
            param.update(request.params)
            if "key" in param:

                key = param.get("key")
                val = param.get("value", None)
                typ = param.get("type", None)
                des = param.get("description", None)

                if val is None:
                    raise ParameterError("Required parameters: value")

                ret = storeConfig(key, val, typ, des)
                string = "setConfig %s" % key
                res[string] = ret

                c.audit['success'] = True
                c.audit['info'] = "%s=%s" % (key, val)

            else:
                # we gather all key value pairs in the conf dict
                conf = {}
                for key in remove_session_from_param(param):
                    val = param.get(key, '') or ''

                    Key = key
                    if not key.startswith('privacyidea'):
                        Key = 'privacyidea.' + key
                    conf[Key] = val

                    string = "setConfig " + key + ":" + val
                    res[string] = True

                    c.audit['success'] = True
                    c.audit['info'] += "%s=%s, " % (key, val)

                updateConfig(conf)

            Session.commit()
            return sendResult(response, res, 1)

        except Exception as exx:  # pragma: no cover
            log.error("error saving config: %r" % exx)
            log.error(traceback.format_exc())
            Session.rollback()
            return sendError(response, exx)

        finally:
            Session.close()

########################################################

    @log_with(log)
    def delConfig(self, action, **params):
        """
        delete a configuration key
        * if an error occurs an exception is serializedsetConfig and returned

        :param key: configuration key name
        :returns: a json result with the deleted value

        """
        res = {}

        try:
            param = getLowerParams(request.params)

            key = getParam(param, "key", required)
            ret = removeFromConfig(key)
            string = "delConfig " + key
            res[string] = ret

            c.audit['success'] = ret
            c.audit['info'] = key

            Session.commit()
            return sendResult(response, res, 1)

        except Exception as exx:  # pragma: no cover
            log.error("error deleting config: %r" % exx)
            log.error(traceback.format_exc())
            Session.rollback()
            return sendError(response, exx)

        finally:
            Session.close()


########################################################
########################################################

    @log_with(log)
    def getConfig(self, action, **params):
        """
        retrieve value of a defined configuration key, or if no key is given,
        the complete configuration is returned
        if an error occurs an exception is serialized and returned

        * remark: the assumption is, that the access to system/getConfig
                  is only allowed to privileged users

        :param key: generic configuration entry name (optional)

        :return: a json result with key value or all key + value pairs

        """
        res = {}
        param = {}
        try:
            param.update(remove_session_from_param(request.params))

            # if there is no parameter, we return them all
            if len(param) == 0:
                conf = get_privacyIDEA_config()
                keys = conf.keys()
                keys.sort()
                for key in keys:
                    if key.startswith("encprivacyidea."):
                        continue
                    if key.startswith("privacyidea."):
                        Key = key[len("privacyidea."):]
                        typ = type(conf.get(key)).__name__
                        if typ == 'datetime':
                            res[Key] = unicode(conf.get(key))
                        else:
                            res[Key] = conf.get(key)

                # as we return the decrypted values, we could do this in place
                # and display the value under the original key
                for key in keys:
                    if key.startswith("encprivacyidea."):
                        Key = key[len("encprivacyidea."):]
                        res[Key] = conf.get(key)

                c.audit['success'] = True
                c.audit['info'] = "complete config"

            else:
                key = getParam(param, "key", required)
                ret = getFromConfig(key)
                string = "getConfig " + key
                res[string] = ret

                c.audit['success'] = ret
                c.audit['info'] = "config key %s" % key

            Session.commit()
            return sendResult(response, res, 1)

        except Exception as exx:  # pragma: no cover
            log.error("error getting config: %r" % exx)
            log.error(traceback.format_exc())
            Session.rollback()
            return sendError(response, exx)

        finally:
            Session.close()


########################################################
    @log_with(log)
    def getRealms(self, action, **params):
        '''
        method:
            system/getRealms

        description:
            returns all realm definitinos as a json result.

        arguments:

        returns:
            a json result with a list of Realms

        exception:
            if an error occurs an exception is serialized and returned


        Either the admin has the policy scope=system, action=read
        or he is rights in scope=admin for some realms.
        If he does not have the system-read-right, then he will only
        see the realms, he is admin of.
        '''

        # config settings from here
        try:
            # param = getLowerParams(request.params)
            realms = getRealms()
            c.audit['success'] = True

            # If the admin is not allowed to see all realms,
            # (policy scope=system, action=read)
            # the realms, where he has no administrative rights need,
            # to be stripped.

            polPost = self.Policy.checkPolicyPost('system',
                                                  'getRealms',
                                                  {'realms': realms})
            res = polPost['realms']

            Session.commit()
            return sendResult(response, res, 1)

        except PolicyException as pex:
            log.error("policy exception: %r" % pex)
            log.error(traceback.format_exc())
            Session.rollback()
            return sendError(response, pex)

        except Exception as exx:  # pragma: no cover
            log.error("error getting realms: %r" % exx)
            log.error(traceback.format_exc())
            Session.rollback()
            return sendError(response, exx)

        finally:
            Session.close()


########################################################
    @log_with(log)
    def setResolver(self, action, **params):
        """
        method:
            system/setResolver

        description:
            creates or updates a useridresolver

        arguments:
            name    -    the name of the resolver
            type    -    the type of the resolver [ldapsersolver, sqlresolver]

            LDAP:
                LDAPURI
                LDAPBASE
                BINDDN
                BINDPW
                TIMEOUT
                SIZELIMIT
                LOGINNAMEATTRIBUTE
                LDAPSEARCHFILTER
                LDAPFILTER
                USERINFO
                NOREFERRALS        - True|False
            SQL:
                Database
                Driver
                Server
                Port
                User
                Password
                Table
                Map

        returns:
            a json result with the found value

        exception:
            if an error occurs an exception is serialized and returned

        """
        res = {}
        param = {}

        try:
            param.update(request.params)

            res = defineResolver(param)

            Session.commit()
            return sendResult(response, res, 1)

        except Exception as exx:  # pragma: no cover
            log.error("error saving config: %r" % exx)
            log.error(traceback.format_exc())
            Session.rollback()
            return sendError(response, exx)

        finally:
            Session.close()


########################################################
    @log_with(log)
    def getResolvers(self, action, **params):
        """
        method:
            system/getResolvers

        descriptions:
            returns a json list of all useridresolvers

        arguments:

        returns:
            a json result with a list of all available resolvers

        exception:
            if an error occurs an exception is serialized and returned

        """
        res = {}

        try:
            res = getResolverList()

            c.audit['success'] = True
            Session.commit()
            return sendResult(response, res, 1)

        except Exception as exx:  # pragma: no cover
            log.error("error getting resolvers: %r" % exx)
            log.error(traceback.format_exc())
            Session.rollback()
            return sendError(response, exx)

        finally:
            Session.close()


########################################################
    @log_with(log)
    def delResolver(self, action, **params):
        """
        method:
            system/delResolver

        description:
            this function deletes an existing resolver
            All config keys of this resolver get deleted

        arguments:
            resolver - the name of the resolver to delete.

        returns:
            success state

        exception:
            if an error occurs an exception is serialized and returned

        """
        res = {}

        try:
            param = getLowerParams(request.params)

            resolver = getParam(param, "resolver", required)
            # only delete a resolver, if it is not used by any realm
            found = False
            fRealms = []
            realms = getRealms()
            for realm in realms:
                info = realms.get(realm)
                reso = info.get('useridresolver')

                for idRes in reso:
                    if resolver == get_resolver_name(idRes):
                        fRealms.append(realm)
                        found = True

            if found is True:
                c.audit['failed'] = res
                err = 'Resolver %r  still in use by the realms: %r' % \
                      (resolver, fRealms)
                c.audit['info'] = err
                raise Exception('%r !' % err)

            res = deleteResolver(resolver)
            c.audit['success'] = res
            c.audit['info'] = resolver

            Session.commit()
            return sendResult(response, res, 1)

        except Exception as exx:  # pragma: no cover
            log.error("error deleting resolver: %r" % exx)
            log.error(traceback.format_exc())
            Session.rollback()
            return sendError(response, exx)

        finally:
            Session.close()


########################################################
    @log_with(log)
    def getResolver(self, action, **params):
        """
        method:
            system/getResolver

        description:
            this function retrieves the definition of the resolver

        arguments:
            resolver - the name of the resolver

        returns:
            a json result with the configuration of a specified resolver

        exception:
            if an error occurs an exception is serialized and returned

        """
        res = {}

        try:
            param = getLowerParams(request.params)

            resolver = getParam(param, "resolver", required)
            if (len(resolver) == 0):
                raise Exception("[getResolver] missing resolver name")

            res = getResolverInfo(resolver)

            c.audit['success'] = True
            c.audit['info'] = resolver

            Session.commit()
            return sendResult(response, res, 1)

        except Exception as exx:  # pragma: no cover
            log.error("error getting resolver: %r" % exx)
            log.error(traceback.format_exc())
            Session.rollback()
            return sendError(response, exx)

        finally:
            Session.close()

    @log_with(log)
    def get_resolver_list(self, action, **params):
        res = {}
        try:
            rlist = getlist()
            res['resolverlist'] = [l for l in rlist]
            res['resolvertypes'] = [l.split(".")[-1] for l in rlist]
            return sendResult(response, res, 1)
        except Exception as exx:
            Session.rollback()
            return sendError(response, exx)
        finally:
            Session.close()

########################################################
    @log_with(log)
    def setDefaultRealm(self, action, **params):
        """
        method:
            system/setDefaultRealm

        description:
            this function sets the given realm to the default realm

        arguments:
            realm - the name of the realm, that should be the default realm

        returns:
            a json result with a list of Realms

        exception:
            if an error occurs an exception is serialized and returned

        """
        res = False

        try:
            param = getLowerParams(request.params)

            defRealm = getParam(param, "realm", optional)
            if defRealm is None:
                defRealm = ""

            defRealm = defRealm.lower().strip()
            res = setDefaultRealm(defRealm)
            if res is False and defRealm != "":
                c.audit['info'] = "The realm %s does not exist" % defRealm

            c.audit['success'] = True
            c.audit['info'] = defRealm

            Session.commit()
            return sendResult(response, res, 1)

        except Exception as exx:  # pragma: no cover
            log.error("setting default realm failed: %r" % exx)
            log.error(traceback.format_exc())
            Session.rollback()
            return sendError(response, exx)

        finally:
            Session.close()


########################################################
    @log_with(log)
    def getDefaultRealm(self, action, **params):
        """
        method:
            system/getDefaultRealm

        description:
            this function returns the default realm

        arguments:
            ./.

        returns:
            a json description of the default realm

        exception:
            if an error occurs an exception is serialized and returned
        """
        res = False

        try:
            defRealm = getDefaultRealm()
            res = getRealms(defRealm)

            c.audit['success'] = True
            c.audit['info'] = defRealm

            Session.commit()
            return sendResult(response, res, 1)

        except Exception as exx:  # pragma: no cover
            log.error("return default realm failed: %r" % exx)
            log.error(traceback.format_exc())
            Session.rollback()
            return sendError(response, exx)

        finally:
            Session.close()


########################################################
    @log_with(log)
    def setRealm(self, action, **params):
        """
        method:
            system/setRealm

        description:
            this function is used to define a realm with the given
            useridresolvers

        arguments:
            realm     - name of the realm
            resolvers - comma seperated list of resolvers, that should be
                        in this realm
                        resolvers need to be specified in the form:
                            privacyidea.lib.resolvers.
                            PasswdIdResolver.IdResolver.<name>

        returns:
            a json result with a list of Realms

        exception:
            if an error occurs an exception is serialized and returned

        """
        res = False
        err = ""
        realm = ""
        param = {}

        try:
            param.update(request.params)

            realm = getParam(param, "realm", required)
            resolvers = getParam(param, "resolvers", required)

            realm_resolvers = []
            for resolver in resolvers.split(','):
                # check resolver returns the correct resolver description
                (res, realm_resolver) = checkResolverType(resolver)
                if res is False:
                    raise Exception("Error in resolver %r please check the"
                                    " logfile!" % resolver)
                realm_resolvers.append(realm_resolver)

            resolvers = ",".join(realm_resolvers)
            res = setRealm(realm, resolvers)
            c.audit['success'] = res
            c.audit['info'] = "realm: %r, resolvers: %r" % (realm, resolvers)

            Session.commit()
            return sendResult(response, res, 1)

        except Exception as exx:  # pragma: no cover
            err = ("Failed to set realm with %r " % param)
            log.error("%r %r" % (err, exx))
            log.error(traceback.format_exc())
            Session.rollback()
            return sendError(response, exx)

        finally:
            Session.close()


########################################################
    @log_with(log)
    def delRealm(self, action, **params):
        """
        method:
            system/delRealm

        description:
            this function deletes the given realm

        arguments:
            realm - the name of the realm to be deleted

        returns:
            a json result if deleting the realm was successful

        exception:
            if an error occurs an exception is serialized and returned

        """
        res = {}

        try:
            param = request.params
            realm = getParam(param, "realm", required)
            # we test if before delete there has been a default
            # if yes - check after delete, if still one there
            #         and set the last available to default
            defRealm = getDefaultRealm()
            hadDefRealmBefore = False
            if defRealm != "":
                hadDefRealmBefore = True

            # now test if realm is defined
            if isRealmDefined(realm) is True:
                if realm.lower() == defRealm.lower():
                    setDefaultRealm("")
                # this is a remnant of linotp 2.0
                if realm == "_default_":  # pragma: no cover
                    realmConfig = "useridresolver"
                else:
                    realmConfig = "useridresolver.group." + realm

                res["delRealm"] = {"result": removeFromConfig(realmConfig,
                                                              iCase=True)}

            ret = deleteRealm(realm)

            if hadDefRealmBefore is True:
                defRealm = getDefaultRealm()
                if defRealm == "":
                    realms = getRealms()
                    if len(realms) == 1:
                        for k in realms:
                            setDefaultRealm(k)
            c.audit['success'] = ret
            c.audit['info'] = realm

            Session.commit()
            return sendResult(response, res, 1)

        except Exception as exx:  # pragma: no cover
            log.error("error deleting realm: %r" % exx)
            log.error(traceback.format_exc())
            Session.rollback()
            return sendError(response, exx)

        finally:
            Session.close()


# #######################################################

    @log_with(log)
    def setPolicy(self, action, **params):
        """
        method:
            system/setPolicy

        description:
            Stores a policy that define ACL or behaviour of several different
            actions in privacyIDEA. The policy is stored as configuration
            values like this:
                Policy.<NAME>.action
                Policy.<NAME>.scope
                Policy.<NAME>.realm

        arguments:
            name:       name of the policy
            action:     which action may be executed
            scope:      selfservice
            realm:      This polcy holds for this realm
            user:       (optional) This polcy binds to this user
            time:       (optional) on which time does this policy hold
            client:     (optional) for which requesting client this should be

        returns:
            a json result with success or error

        exception:
            if an error occurs an exception is serialized and returned

        """
        res = {}
        param = {}
        try:
            param.update(remove_session_from_param(request.params))

            name = getParam(param, "name", required)

            # check that the name does not contain a .
            if not re.match('^[a-zA-Z0-9_]*$', name):
                raise Exception(_("The name of the policy may only contain "
                                  "the characters a-zA-Z0-9_"))
            if not name:
                raise Exception(_("The name of the policy must not be empty"))

            action = getParam(param, "action", required)
            scope = getParam(param, "scope", required)
            realm = getParam(param, "realm", required)
            user = getParam(param, "user", optional)
            time = getParam(param, "time", optional)
            client = getParam(param, "client", optional)
            active = getParam(param, "active", optional)

            p_param = {'name': name,
                       'action': action,
                       'scope': scope,
                       'realm': realm,
                       'user': user,
                       'time': time,
                       'client': client,
                       'active': active}

            c.audit['action_detail'] = unicode(param)

            if len(name) > 0 and len(action) > 0:

                log.debug("saving policy %r" % p_param)
                ret = setPolicy(p_param)
                log.debug("policy %s successfully saved." % name)

                string = "setPolicy " + name
                res[string] = ret

                c.audit['success'] = True

                Session.commit()
            else:
                log.error("failed: policy with empty name or action %r"
                          % p_param)
                string = "setPolicy <%r>" % name
                res[string] = False

                c.audit['success'] = False
                raise Exception('setPolicy failed: name and action required!')

            return sendResult(response, res, 1)

        except Exception as exx:  # pragma: no cover
            log.error("error saving policy: %r" % exx)
            log.error(traceback.format_exc())
            Session.rollback()
            return sendError(response, exx)

        finally:
            Session.close()


########################################################
    @log_with(log)
    def policies_flexi(self, action, **params):
        '''
        This function is used to fill the policies tab
        Unlike the complex /system/getPolcies function, it only returns a
        simple array of the tokens.
        '''

        pol = {}

        try:
            param = getLowerParams(request.params)

            name = getParam(param, "name", optional)
            realm = getParam(param, "realm", optional)
            scope = getParam(param, "scope", optional)
            sortname = getParam(param, "sortname", optional)
            sortorder = getParam(param, "sortorder", optional)

            log.debug("retrieving policy name: %s, realm: %s, scope: %s, "
                      "sort:%s by %s"
                      % (name, realm, scope, sortorder, sortname))
            pols = self.Policy.getPolicy({'name': name,
                                          'realm': realm,
                                          'scope': scope},
                                         display_inactive=True)

            lines = []
            for pol in pols:
                lines.append({'id': pol,
                              'cell': [1 if pols[pol].get('active',
                                                          "True") == "True"
                                       else 0,
                                       pol,
                                       pols[pol].get('user', ""),
                                       pols[pol].get('scope', ""),
                                       escape(pols[pol].get('action', "")
                                              or ""),
                                       pols[pol].get('realm', ""),
                                       pols[pol].get('client', ""),
                                       pols[pol].get('time', "")
                                       ]
                              }
                             )
            # sorting
            reverse = False
            sNames = {'active': 0,
                      'name': 1,
                      'user': 2,
                      'scope': 3,
                      'action': 4,
                      'realm': 5,
                      'client': 6,
                      'time': 7}
            if sortorder == "desc":
                reverse = True
            lines = sorted(lines,
                           key=lambda policy: policy['cell'][sNames[sortname]],
                           reverse=reverse)
            # end: sorting

            # We need to return 'page', 'total', 'rows'
            res = {"page": 1,
                   "total": len(lines),
                   "rows": lines}

            c.audit['success'] = True
            c.audit['info'] = "name = %s, realm = %s, scope = %s" % (name,
                                                                     realm,
                                                                     scope)

            Session.commit()
            response.content_type = 'application/json'
            return json.dumps(res, indent=3)

        except Exception as exx:  # pragma: no cover
            log.error("error in policy flexi: %r" % exx)
            log.error(traceback.format_exc())
            Session.rollback()
            return sendError(response, exx)

        finally:
            Session.close()


########################################################
    @log_with(log)
    def getPolicyDef(self, action, **params):
        '''
        method:
            system/getPolicyDef

        description:
            This is a helper function that returns the POSSIBLE policy
            definitions, that can
            be used to define your policies.

        arguments:
            scope - optional - if given, the function will only return policy
            definitions for the given scope.

        returns:
             the policy definitions of
              - allowed scopes
              - allowed actions in scopes
              - type of actions

        exception:
            if an error occurs an exception is serialized and returned
        '''
        pol = {}

        try:
            param = getLowerParams(request.params)
            log.debug("getting policy definitions: %r" % param)

            scope = getParam(param, "scope", optional)
            pol = get_policy_definitions(scope)
            dynpol = self._add_dynamic_tokens(scope)
            pol.update(dynpol)

            c.audit['success'] = True
            c.audit['info'] = scope

            Session.commit()
            return sendResult(response, pol, 1)

        except Exception as exx:  # pragma: no cover
            log.error("error getting policy definitions: %r" % exx)
            log.error(traceback.format_exc())
            Session.rollback()
            return sendError(response, exx)

        finally:
            Session.close()


#########################################################
    @log_with(log)
    def _add_dynamic_tokens(self, scope):
        '''
            add the policy description of the dynamic token

            :param scope: scope of the policy definition
            :type  scope: string

            :return: policy dict
            :rtype:  dict

        '''
        pol = {}

        glo = config['pylons.app_globals']
        tokenclasses = glo.tokenclasses

        for tok in tokenclasses.keys():
            tclass = tokenclasses.get(tok)
            tclass_object = newToken(tclass)
            if hasattr(tclass_object, 'getClassInfo'):
                # check if we have a policy in the definition
                try:
                    policy = tclass_object.getClassInfo('policy', ret=None)
                    if policy is not None and scope in policy:
                        scope_policy = policy.get(scope)
                        pol.update(scope_policy)
                except Exception as exx:
                    log.info('no policy for tokentype %r found (%r)'
                             % (tok, exx))

        return pol

#########################################################
    @log_with(log)
    def importPolicy(self, action, **params):
        '''
        method:
            system/importPolicy

        description:
            This function is used to import policies from a file.

        arguments:
            file - mandatory: The policy file in the POST request
        '''
        sendResultMethod = sendResult
        sendErrorMethod = sendError

        res = True
        try:
            policy_file = request.POST['file']
            fileString = ""
            log.debug("loading policy file to server using POST request. "
                      "File: %s" % policy_file)

            # In case of form post requests, it is a "instance" of FieldStorage
            # i.e. the Filename is selected in the browser and the data is
            # transferred
            # in an iframe. see: http://jquery.malsup.com/form/#sample4
            #
            if type(policy_file).__name__ == 'instance':
                log.debug("Field storage file: %s", policy_file)
                fileString = policy_file.value
                sendResultMethod = sendXMLResult
                sendErrorMethod = sendXMLError
            else:
                fileString = policy_file
            log.debug("fileString: %s", fileString)

            if fileString == "":
                log.error("Error loading/importing policy file. file empty!")
                return sendErrorMethod(response, "Error loading policy. "
                                       "File empty!")

            # the contents of filestring needs to be parsed and stored as
            # policies.
            from configobj import ConfigObj
            policies = ConfigObj(fileString.split('\n'), encoding="UTF-8")
            log.info("read the following policies: %s" % policies)
            res = len(policies)
            for policy_name in policies.keys():
                ret = setPolicy({'name': policy_name,
                                 'action': policies[policy_name].get('action',
                                                                     ""),
                                 'scope': policies[policy_name].get('scope',
                                                                    ""),
                                 'realm': policies[policy_name].get('realm',
                                                                    ""),
                                 'user': policies[policy_name].get('user', ""),
                                 'time': policies[policy_name].get('time', ""),
                                 'client': policies[policy_name].get('client',
                                                                     "")
                                 })
                log.debug("import policy %s: %s" % (policy_name, ret))

            c.audit['info'] = "Policies imported from file %s" % policy_file
            c.audit['success'] = 1
            Session.commit()
            return sendResultMethod(response, res)

        except Exception as exx:  # pragma: no cover
            log.error("failed! %r" % exx)
            log.error(traceback.format_exc())
            Session.rollback()
            return sendErrorMethod(response, exx)

        finally:
            Session.close()


############################################################
    @log_with(log)
    def checkPolicy(self, action, **params):
        '''
        method:
            system/checkPolicy

        description:
            this function checks if a the given parameter will trigger
            a policy or not.

        arguments:
            * user   - the name of the user
            * realm  - the realm
            * scope  - the scope
            * action
            * client - the client IP

        returns:
            a json result like this:
              value : { "allowed" : "true",
                        "policy" : <Name der Policy, die das erlaubt hat> }
              value : { "allowed" : "false",
                         "info" : <sowas wie die Fehlermeldung> }

        '''
        res = {}

        try:
            param = getLowerParams(request.params)

            user = getParam(param, "user", required)
            realm = getParam(param, "realm", required)
            scope = getParam(param, "scope", required)
            action = getParam(param, "action", required)
            client = getParam(param, "client", required)

            pol = {}
            if scope in ["admin", "system", "license"]:
                pol = self.Policy.getPolicy({"scope": scope})
                if len(pol) > 0:
                    # Policy active for this scope!
                    pol = self.Policy.getPolicy({"user": user,
                                                 "realm": realm,
                                                 "scope": scope,
                                                 "action": action,
                                                 "client": client})
                    res["allowed"] = len(pol) > 0
                    res["policy"] = pol
                    if len(pol) > 0:
                        c.audit['info'] = "allowed by policy %s" % pol.keys()
                else:
                    # No policy active for this scope
                    c.audit['info'] = "allowed since no policies in scope %s"\
                                      % scope
                    res["allowed"] = True
                    res["policy"] = "No policies in scope %s" % scope
            else:
                log.debug("checking policy for client %s, scope %s, action %s,"
                          " realm %s and user %s" %
                          (client, scope, action, realm, user))

                pol = self.Policy.get_client_policy(client,
                                                    scope,
                                                    action,
                                                    realm,
                                                    user)
                res["allowed"] = len(pol) > 0
                res["policy"] = pol
                if len(pol) > 0:
                    c.audit['info'] = "allowed by policy %s" % pol.keys()

            c.audit['action_detail'] = "action = %s, realm = %s, scope = %s"\
                                       % (action, realm, scope)
            c.audit['success'] = True

            Session.commit()
            return sendResult(response, res, 1)

        except Exception as exx:  # pragma: no cover
            log.error("error checking policy: %r" % exx)
            log.error(traceback.format_exc())
            Session.rollback()
            return sendError(response, exx)

        finally:
            Session.close()

##########################################################################
    @log_with(log)
    def getPolicy(self, action, **params):
        """
        method:
            system/getPolicy

        description:
            this function is used to retrieve the policies that you
            defined.

        arguments:
            * realm - (optional) will return all policies in the given realm
            * name  - (optional) will only return the policy with the
                                 given name
            * scope - (optional) will only return the policies within the
                                 given scope
            * export - (optional) The filename needs to be specified as the
                                  third part of the URL like
                                  /system/getPolicy/policy.cfg. It
                                  will then be exported to this file.
            * display_inactive - (optional) if set, then also inactive
                                 policies will be displayed

        returns:
            a json result with the configuration of the specified policies

        exception:
            if an error occurs an exception is serialized and returned

        """

        pol = {}
        param = getLowerParams(request.params)
        export = None

        # config settings from here

        try:
            name = getParam(param, "name", optional)
            realm = getParam(param, "realm", optional)
            scope = getParam(param, "scope", optional)
            disp_inactive = getParam(param, "display_inactive", optional)
            if disp_inactive:
                disp_inactive = True

            route_dict = request.environ.get('pylons.routes_dict')
            export = route_dict.get('id')

            log.debug("retrieving policy name: %s, realm: %s, scope: %s"
                      % (name, realm, scope))
            pol = {}
            if name is not None:
                for nam in name.split(','):
                    p = self.Policy.getPolicy({'name': nam,
                                               'realm': realm,
                                               'scope': scope},
                                              display_inactive=disp_inactive)
                    pol.update(p)
            else:
                pol = self.Policy.getPolicy({'name': name,
                                             'realm': realm,
                                             'scope': scope},
                                            display_inactive=disp_inactive)

            c.audit['success'] = True
            c.audit['info'] = "name = %s, realm = %s, scope = %s" \
                              % (name, realm, scope)

            Session.commit()

            if export:
                filename = self.Policy.create_policy_export_file(pol, export)
                wsgi_app = FileApp(filename)
                return wsgi_app(request.environ, self.start_response)
            else:
                return sendResult(response, pol, 1)

        except Exception as exx:  # pragma: no cover
            log.error("error getting policy: %r" % exx)
            log.error(traceback.format_exc())
            Session.rollback()
            return sendError(response, exx)

        finally:
            Session.close()

########################################################
    @log_with(log)
    def delPolicy(self, action, **params):
        """
        method:
            system/delPolicy

        description:
            this function deletes the policy with the given name

        arguments:
            name  - the policy with the given name

        returns:
            a json result about the delete success

        exception:
            if an error occurs an exception is serialized and returned

        """
        res = {}
        try:
            param = getLowerParams(request.params)
            name = getParam(param, "name", required)
            log.debug("trying to delete policy %s" % name)
            ret = deletePolicy(name)
            res["delPolicy"] = {"result": ret}

            c.audit['success'] = ret
            c.audit['info'] = name

            Session.commit()
            return sendResult(response, res, 1)

        except Exception as exx:  # pragma: no cover
            log.error("error deleting policy: %r" % exx)
            log.error(traceback.format_exc())
            Session.rollback()
            return sendError(response, exx)

        finally:
            Session.close()


########################################################
    @log_with(log)
    def setupSecurityModule(self, action, **params):

        res = {}

        try:
            params = getLowerParams(request.params)
            hsm_id = params.get('hsm_id', None)

            from privacyidea.lib.config import getGlobalObject
            glo = getGlobalObject()
            sep = glo.security_provider

            # for test purpose we switch to an errHSM
            if isSelfTest():
                if params.get('__hsmexception__') == '__ON__':
                    hsm = c.hsm.get('obj')
                    hsm_id = sep.activeOne
                    if type(hsm).__name__ == 'DefaultSecurityModule':
                        hsm_id = sep.setupModule('err', params)

                if params.get('__hsmexception__') == '__OFF__':
                    hsm = c.hsm.get('obj')
                    hsm_id = sep.activeOne
                    if type(hsm).__name__ == 'ErrSecurityModule':
                        hsm_id = sep.setupModule('default', params)

            if hsm_id is None:
                hsm_id = sep.activeOne
                hsm = c.hsm.get('obj')
                error = c.hsm.get('error')
                if hsm is None or len(error) != 0:
                    raise Exception('current activeSecurityModule >%r< '
                                    'is not initialized::%s:: - Please '
                                    'check your security module configuration'
                                    ' and connection!' % (hsm_id, error))

                ready = hsm.isReady()
                res['setupSecurityModule'] = {'activeSecurityModule': hsm_id,
                                              'connected': ready}
                ret = ready
            else:
                if hsm_id != sep.activeOne:
                    raise Exception('current activeSecurityModule >%r< could '
                                    'only be changed through the '
                                    'configuration!' % sep.activeOne)

                ret = sep.setupModule(hsm_id, config=params)

                hsm = c.hsm.get('obj')
                ready = hsm.isReady()
                res['setupSecurityModule'] = {'activeSecurityModule': hsm_id,
                                              'connected': ready,
                                              'result': ret}

            c.audit['success'] = ret
            Session.commit()
            return sendResult(response, res, 1)

        except Exception as exx:  # pragma: no cover
            log.error("setup failed: %r" % exx)
            log.error(traceback.format_exc())
            Session.rollback()
            return sendError(response, exx)

        finally:
            Session.close()

# eof###########################################################################


# -*- coding: utf-8 -*-
#  privacyIDEA is a fork of LinOTP
#  May 08, 2014 Cornelius Kölbel
#  License:  AGPLv3
#  contact:  http://www.privacyidea.org
#
#  Copyright (C) 2010 - 2014 LSE Leading Security Experts GmbH
#  License:  AGPLv3
#  contact:  http://www.linotp.org
#            http://www.lsexperts.de
#            linotp@lsexperts.de
#
# This code is free software; you can redistribute it and/or
# modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
# License as published by the Free Software Foundation; either
# version 3 of the License, or any later version.
#
# This code is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU AFFERO GENERAL PUBLIC LICENSE for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
'''
Description:  functional tests
                
Dependencies: -
'''

import logging
from privacyidea.tests import TestController, url
from webtest.app import AppError

log = logging.getLogger(__name__)

class TestAuditController(TestController):



    def test_audit_search(self):
        response = self.app.get(url(controller='audit', action='search'), params={})
        print response
        self.assertTrue('page": 1,' in response)
        
        response = self.app.get(url(controller='audit', action='search'), params={"sortname" : "serial",
                                                                                  "outform" : "csv"})
        print response
        self.assertTrue(' null, "INFO", "0"' in response)
        



# -*- coding: utf-8 -*-
#
#  privacyIDEA is a fork of LinOTP
#  May 08, 2014 Cornelius Kölbel
#  License:  AGPLv3
#  contact:  http://www.privacyidea.org
#
#  Copyright (C) 2010 - 2014 LSE Leading Security Experts GmbH
#  License:  AGPLv3
#  contact:  http://www.linotp.org
#            http://www.lsexperts.de
#            linotp@lsexperts.de
#
# This code is free software; you can redistribute it and/or
# modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
# License as published by the Free Software Foundation; either
# version 3 of the License, or any later version.
#
# This code is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU AFFERO GENERAL PUBLIC LICENSE for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
import logging


from pylons             import tmpl_context as c
from privacyidea.lib.base    import BaseController
from pylons.templating  import render_mako as render
from privacyidea.lib.util    import get_version
from privacyidea.lib.util    import get_copyright_info
from privacyidea.lib.reply import sendError

from pylons import response
from privacyidea.model.meta import Session
from privacyidea.lib.log import log_with

import traceback

log = logging.getLogger(__name__)

optional = True
required = False


class AuthController(BaseController):


    @log_with(log)
    def __before__(self, action, **params):

        try:

            c.version = get_version()
            c.licenseinfo = get_copyright_info()
            self.set_language()

        except Exception as exx:
            log.error("%r exception %r" % (action, exx))
            log.error(traceback.format_exc())
            Session.rollback()
            Session.close()
            return sendError(response, exx, context='before')

        finally:
            pass


    def index(self):
        '''
        This is the method for testing authentication

        Call it directly in your browser like this
            http(s)://server/auth/index
        '''
        log.debug("index, authenticating user")
        return render("/auth.mako")


    def index3(self):
        '''
        This is the method for testing authentication

        Call it directly in your browser like this
            http(s)://server/auth/index3
        '''
        log.debug("index, authenticating user")
        return render("/auth3.mako")


    def requestsms(self):
        '''
        This is the method for requesting the sending of SMS

        Call it directly in your browser like this
            http(s)://server/auth/requestsms
        '''
        log.debug("authenticating user")
        return render("/auth-sms.mako")


    def ocra(self):
        '''
        This is the method for testing ocra tokens

        Call it directly in your browser like this
            http(s)://server/auth/ocra
        '''
        log.debug("authenticating user")
        return render("/auth-ocra.mako")


    def ocra2(self):
        '''
        This is the method for testing ocra2 tokens

        Call it directly in your browser like this
            http(s)://server/auth/ocra2
        '''
        log.debug("authenticating user")
        return render("/auth-ocra2.mako")


#eof##########################################################################




# -*- coding: utf-8 -*-
#  privacyIDEA is a fork of LinOTP
#  May 08, 2014 Cornelius Kölbel
#  License:  AGPLv3
#  contact:  http://www.privacyidea.org
#
#  2014-10-04 Cornelius Kölbel <cornelius@privacyidea.org>
#             Some PEP8 stuff.
#
#  Copyright (C) 2010 - 2014 LSE Leading Security Experts GmbH
#  License:  AGPLv3
#  contact:  http://www.linotp.org
#            http://www.lsexperts.de
#            linotp@lsexperts.de
#
# This code is free software; you can redistribute it and/or
# modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
# License as published by the Free Software Foundation; either
# version 3 of the License, or any later version.
#
# This code is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU AFFERO GENERAL PUBLIC LICENSE for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
'''
  Description:  functional tests
                
  Dependencies: -

'''
from privacyidea.lib.resolvers.UserIdResolver import getResolverClass

from privacyidea.tests import TestController


import logging
log = logging.getLogger(__name__)
import os


class TestPasswdController(TestController):
    
    def setUp(self):
        TestController.setUp(self)
        self.serials = []
        self.cwd = os.getcwd()

    def test_resolver(self):
        '''
        Testing PasswdIdResolver
        '''
        y = getResolverClass("PasswdIdResolver", "IdResolver")()
        y.loadConfig({'privacyidea.passwdresolver.fileName': '%s/privacyidea'
                      '/tests/testdata/my-passwd' % self.cwd},
                     "")

        userlist = y.getUserList({'username': '*',
                                  "userid": "= 1000"})
        print userlist
        assert userlist[0].get('username') == "heinz"

        loginId = y.getUserId("heinz")
        print loginId
        assert loginId == '1000'

        ret = y.getUserInfo(loginId)
        print ret
        assert ret.get('username') == "heinz"

        username_exists = y.getUsername('1000')
        print "Username exists: %r" % username_exists
        assert username_exists == "heinz"

    def test_no_file(self):
        '''
        Testing PasswdIdResolver without file
        '''
        y = getResolverClass("PasswdIdResolver", "IdResolver")()
        y.loadFile()

        userlist = y.getUserList({'username': '*',
                                  "userid": "= 0"})
        print userlist
        assert userlist[0].get('username') == "root"

        loginId = y.getUserId("root")
        print loginId
        assert loginId == '0'

        ret = y.getUserInfo(loginId)
        print ret
        assert ret.get('username') == "root"

    def test_checkpass_shadow(self):
        '''
        Testing checkpass with PasswdIdResolver with a shadow passwd file
        '''
        y = getResolverClass("PasswdIdResolver", "IdResolver")()
        y.loadConfig({'privacyidea.passwdresolver.fileName': '%s/privacyidea'
                      '/tests/testdata/my-passwd' % self.cwd},
                     "")

        success = False
        try:
            y.checkPass('1000', "geheim")
        except NotImplementedError:
            success = True

        assert success

    def test_checkpass(self):
        '''
        Testing checkpass
        '''
        y = getResolverClass("PasswdIdResolver", "IdResolver")()
        y.loadConfig({"privacyidea.passwdresolver.fileName": "%s/privacyidea"
                      "/tests/testdata/my-pass2" % self.cwd},
                     "")

        res = y.checkPass('2001', "geheim")
        print "result %r" % res
        assert res

        res = y.checkPass('2001', "wrongPW")
        print "result %r" % res
        assert res is False

    def test_searchfields(self):
        '''
        Testing getSearchfields
        '''
        y = getResolverClass("PasswdIdResolver", "IdResolver")()
        y.loadConfig({'privacyidea.passwdresolver.fileName': '%s/privacyidea'
                      '/tests/testdata/my-pass2' % self.cwd},
                     "")

        s = y.getSearchFields()
        print s
        assert s



# -*- coding: utf-8 -*-
#  privacyIDEA is a fork of LinOTP
#  May 08, 2014 Cornelius Kölbel
#  License:  AGPLv3
#  contact:  http://www.privacyidea.org
#
#  Copyright (C) 2010 - 2014 LSE Leading Security Experts GmbH
#  License:  AGPLv3
#  contact:  http://www.linotp.org
#            http://www.lsexperts.de
#            linotp@lsexperts.de
#
# This code is free software; you can redistribute it and/or
# modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
# License as published by the Free Software Foundation; either
# version 3 of the License, or any later version.
#
# This code is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU AFFERO GENERAL PUBLIC LICENSE for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with this program.  If not, see <http://www.gnu.org/licenses/>.
#  
'''
    OcraSuite = CryptoFunction(K, DataInput)

    CryptoFunction = HOTP-H-n
        H in { SHA-1, SHA256, SHA512 }
        n - truncation 4-10 or 0 (= no truncation)

    DataInput = OCRASuite | 00 | C | Q | P | S | T
        OCRASuite - mode of operation
        C - Counter (optional)
        Q - Challenge
        P - hashed password/pin (optional)
        S - session information (optional)
        T - timestamp (optional)


   [C] | QFxx | [PH | Snnn | TG] : Challenge-Response computation

   [C] | QFxx | [PH | TG] : Plain Signature computation

   Each input that is used for the computation is represented by a
   single letter (except Q) as and they are separated by a hyphen.

   + Q: Challenge +
   The input for challenge is further qualified by the formats supported
   by the client for challenge question(s).  Supported values can be:

                 +------------------+-------------------+
                 |    Format (F)    | Up to Length (xx) |
                 +------------------+-------------------+
                 | A (alphanumeric) |       04-64       |
                 |    N (numeric)   |       04-64       |
                 |  H (hexadecimal) |       04-64       |
                 +------------------+-------------------+

                      Table 2: Challenge Format Table

   The default challenge format is N08, numeric and up to 8 digits.

   + P: Pin +
   The input for P is further qualified by the hash function used for
   the PIN/password.  Supported values for hash function can be:

   Hash function (H) - SHA1, SHA256, SHA512.

   The default hash function for P is SHA1.

   + S: Session +
   The input for S is further qualified by the length of the session
   data in bytes.  The client and server could agree to any length but
   the typical values are:

   Length (nnn) - 064, 128, 256, and 512.

   The default length is 064 bytes.

   The input for timestamps is further qualified by G, size of the time-
   step.  G can be specified in number of seconds, minutes, or hours:

           +--------------------+------------------------------+
           | Time-Step Size (G) |           Examples           |
           +--------------------+------------------------------+
           |       [1-59]S      | number of seconds, e.g., 20S |
           |       [1-59]M      |  number of minutes, e.g., 5M |
           |       [0-48]H      |  number of hours, e.g., 24H  |
           +--------------------+------------------------------+

                       Table 3: Time-step Size Table

   Default value for G is 1M, i.e., time-step size is one minute and the
   T represents the number of minutes since epoch time [UT].


    OCRASuite = 'Algorithm:CryptoFunction:DataInput'

    "OcraSuite-1:HOTP-SHA512-8:C-QN08-PSHA1"
        OcraSuite-1: OcraSuite Version 1
        HOTP-SHA512-8: HTOP mit SHA512 verkuerzt auf 8 Ziffern
        C-QN08-PSHA1:
            C: mit counter
            QN08: numerische Challenge bis zu 8 Ziffern
            PSHA1: SHA1 des Passworts

    "OcraSuite-1:HOTP-SHA256-6:QA10-T1M"
        OcraSuite-1: OcraSuite Version 1
        HOTP-SHA256-6: HTOP mit SHA256 verkuerzt auf 6 Ziffern
        QA10-T1M:
            QA10: alphanumerische Challenge bis zu 10 Zeichen.
            T1M: Timestamp Counter (time step = 1 minute)


'ocrasuite' in unit tests:

OcraSuite-1:HOTP-SHA1-6:QN08
OcraSuite-1:HOTP-SHA256-8:QA08
OcraSuite-1:HOTP-SHA256-8:QN08-PSHA1

OcraSuite-1:HOTP-SHA512-8:C-QN08
OcraSuite-1:HOTP-SHA256-8:C-QN08-PSHA1

OcraSuite-1:HOTP-SHA512-8:QN08-T1M
OcraSuite-1:HOTP-SHA512-8:QA10-T1M




When computing a response, the concatenation order is always the
following:

    C | OTHER-PARTY-GENERATED-CHALLENGE-QUESTION |  YOUR-GENERATED-CHALLENGE-QUESTION | P| S | T

If a value is empty (i.e., a certain input is not used in the
computation) then the value is simply not represented in the string.

The counter on the token or client MUST be incremented every time a
new computation is requested by the user.  The server's counter value
MUST only be incremented after a successful OcraSuite authentication.




              CLIENT                                   SERVER
             (PROVER)                                 VERIFIER)
                |                                        |
                |   Verifier sends challenge to prover   |
                |   Challenge = Q                        |
                |<---------------------------------------|
                |                                        |
                |   Prover Computes Response             |
                |   R = OcraSuite(K, {[C] | Q | [P | S | T]}) |
                |   Prover sends Response = R            |
                |--------------------------------------->|
                |                                        |
                |  Verifier Validates Response           |
                |  If Response is valid, Server sends OK |
                |  If Response is not,  Server sends NOK |
                |<---------------------------------------|
                |                                        |


          CLIENT                                             SERVER
        (PROVER)                                          (VERIFIER)
           |                                                  |
           |   1. Client sends client-challenge               |
           |   QC = Client-challenge                          |
           |------------------------------------------------->|
           |                                                  |
           |   2. Server computes server-response             |
           |      and sends server-challenge                  |
           |   RS = OcraSuite(K, [C] | QC | QS | [S | T])          |
           |   QS = Server-challenge                          |
           |   Response = RS, QS                              |
           |<-------------------------------------------------|
           |                                                  |
           |   3. Client verifies server-response             |
           |      and computes client-response                |
           |   OcraSuite(K, [C] | QC | QS | [S | T]) != RS -> STOP |
           |   RC = OcraSuite(K, [C] | QS | QC | [P | S | T])      |
           |   Response = RC                                  |
           |------------------------------------------------->|
           |                                                  |
           |   4. Server verifies client-response             |
           |   OcraSuite(K, [C] | QS | QC | [P|S|T]) != RC -> STOP |
           |   Response = OK                                  |
           |<-------------------------------------------------|
           |                                                  |

'''


import binascii
from datetime import datetime
import hashlib
import hmac
import re
from privacyidea.lib.crypto import urandom
from privacyidea.lib.log import log_with
## for the hmac algo, we have to check the python version
import sys
(ma, mi, _, _, _,) = sys.version_info
pver = float(int(ma) + int(mi) * 0.1)


import logging
log = logging.getLogger(__name__)


def is_int(v):
    try:
        int(v)
        return True
    except ValueError:
        return False

def truncated_value(h):
    bytes = map(ord, h)
    offset = bytes[-1] & 0xf
    v = (bytes[offset] & 0x7f) << 24 | (bytes[offset + 1] & 0xff) << 16 | \
            (bytes[offset + 2] & 0xff) << 8 | (bytes[offset + 3] & 0xff)
    return v

def dec(h, p):
    v = unicode(truncated_value(h))
    if len(v) < p: v = (p - len(v)) * "0" + v
    return v[len(v) - p:]

def int2beint64(i):
    hex_counter = hex(long(i))[2:-1]
    hex_counter = '0' * (16 - len(hex_counter)) + hex_counter
    bin_counter = binascii.unhexlify(hex_counter)
    return bin_counter

def bytearray_to_bytes(a_bytearray):
    return bytes([a_byte for a_byte in a_bytearray])

PERIODS = { 'H': 3600, 'M': 60, 'S': 1 }



class OcraSuite():
    '''
    OCRA-1:HOTP-SHA1-6:QN08
    OCRA-1:HOTP-SHA256-8:QA08
    OCRA-1:HOTP-SHA256-8:QN08-PSHA1

    OCRA-1:HOTP-SHA512-8:C-QN08
    OCRA-1:HOTP-SHA256-8:C-QN08-PSHA1

    OCRA-1:HOTP-SHA512-8:QN08-T1M
    OCRA-1:HOTP-SHA512-8:QA10-T1M
    '''
    def __init__(self, ocrasuite, secretObject=None):

        self.secretObj = secretObject
        self.C = None
        self.Q = None
        self.P = None
        self.S = None
        self.T = None

        self.ocrasuite_description = ocrasuite
        (version, crypto, caller) = ocrasuite.split(':')

        ## version check
        if version.upper() != 'OCRA-1':
            raise Exception('unsupported ocra version')

        ## crypto algo
        (hotpStr, hash, trunc) = crypto.split('-')

        if hotpStr.upper() != 'HOTP':
            raise Exception('unsupported hash version: %s' % (unicode(hotpStr)))

        self.hash_algo = self._getCrypto(hash.lower())
        self.truncation = self._getTruncation(trunc)

        ## communication QA10-T1M or C-QN08-PSHA1 . . . QA99??
        params = caller.split('-')
        for param in params:
            ## set and verify the counter
            if param[0] == 'C':
                self.C = param

            elif param[0] == 'Q':
                ## verify the challenge description
                self.Q = ('N', 8)
                if len(param[1:]) > 0:
                    if param[1] not in 'ANH':
                        raise ValueError
                    length = int(param[2:])
                    ## the spec says only max 64
                    if length < 4 or length > 64:
                        raise ValueError
                self.Q = (param[1], length)

            elif param[0] == 'S':
                ## verify and set session description
                self.S = 64
                S = param[1:]
                if S not in ['064', '128', '256', '512']:
                    raise ValueError, ('Unknown session length %s' % (S))
                self.S = int(S)

            elif param[0] == 'P':
                ## verify and set Pin hash algo
                self.P = self._getCrypto(param[1:] or 'SHA1')

            elif param[0] == 'T':
                ## verify and set timestep parameter
                complement = param[1:] or '1M'
                try:
                    length = 0
                    if not re.match('^(\d+[HMS])+$', complement):
                        raise ValueError
                    parts = re.findall('\d+[HMS]', complement)
                    for part in parts:
                        period = part[-1]
                        quantity = int(part[:-1])
                        length += quantity * PERIODS[period]
                    self.T = length
                except ValueError:
                    raise ValueError, ('Invalid timestamp descriptor', complement)


    def compute(self, data, key=None):
        '''
           Compute an HOTP digest using the given key and data input and
           following the current crypto function description.
        '''
        h_data = binascii.hexlify(data)
        try:
            data_input = bytearray(str(self.ocrasuite_description + '\0'))
            for d in data:
                data_input.append(d)

        except Exception as e:
            log.error('Failed to encode data %r: \n%r' % (e, h_data))

        ## call the secret object to get the object
        ##    convert it to binary
        ##                        from privacyidea.lib.crypto import SecretObj
        h = None

        if pver <= 2.6:
            data_input = str(data_input)

        if self.secretObj is not None:
            h = self.secretObj.hmac_digest(data_input, self.hash_algo)
        else:
            bkey = key
            '''' akey = binascii.hexlify(bkey) '''
            h = hmac.new(bkey, data_input, self.hash_algo).digest()

        if self.truncation:
            ret = dec(h, self.truncation)
        else:
            ret = str(truncated_value(h))

        return ret

    def signData(self, data, key=None):
        h = None
        if key is None:
            h = self.secretObj.hmac_digest(data, self.hash_algo)
        else:
            h = hmac.new(key, data, self.hash_algo).digest()
        h_out = binascii.hexlify(h)

        return h_out

    def _getCrypto(self, description):
        '''
           Convert the name of a hash algorithm as described in the OATH
           specifications, to a python object handling the digest algorithm
           interface
        '''
        algo = getattr(hashlib, description.lower(), None)
        if not callable(algo):
            raise ValueError, ('Unknown hash algorithm', description)
        return algo

    def _getTruncation(self, trunc):
        truncation_length = 8
        try:
            truncation_length = int(trunc)
            if truncation_length < 0 or truncation_length > 10:
                raise ValueError
        except ValueError:
            raise ValueError, ('Invalid truncation length', trunc)
        return truncation_length

###############################################################################
## runtime method
###############################################################################
    def combineData(self, C=None, Q=None, P=None, P_digest=None, S=None, T=None, T_precomputed=None, Qsc=None):
        datainput = ''

        if self.C is not None:
            datainput += self._addCounter(C)
        if Q is not None:
            datainput += self._addChallenge(Q)
        if self.P is not None:
            datainput += self._addPin(P, P_digest)
        if self.S is not None:
            datainput += self._addSession(S)
        if self.T is not None:
            datainput += self._addTimeStr(T, T_precomputed)
        #log.error('datainput: %s' % binascii.hexlify(datainput))

        return datainput

    def _addCounter(self, C):
        datainput = ''
        if self.C:
            try:
                C = int(C)
                if C < 0 or C > 2 ** 64:
                    raise Exception()
            except:
                raise ValueError, ('Invalid counter value', C)
            datainput = int2beint64(int(C))
        return datainput

    def _addChallenge(self, Q):
        datainput = ''

        if self.Q:
            max_length = self.Q[1]
            ## do some sanity checks
            if Q is None or len(Q) == 0:
                raise ValueError('challenge is empty : %s' % (str(Q)))
            if type(Q) == unicode:
                ## this might raise an ascii conversion exception
                Q = str(Q)
            if not isinstance(Q, str):
                raise ValueError('challenge is no string: %s' % (str(Q)))
            if len(Q) > max_length:
                raise ValueError('challenge is to long: %s' % (str(Q)))

            if self.Q[0] == 'N' and not Q.isdigit():
                raise ValueError('challenge is not digits only: %s' % (Q))
            if self.Q[0] == 'A' and not Q.isalnum():
                raise ValueError('challenge is not alpha-numeric: %s' % (Q))
            if self.Q[0] == 'H':
                try:
                    int(Q, 16)
                except ValueError:
                    raise ValueError('challenge is hex only: %s' % (Q))

            ## now encode the challenge acordingly
            if self.Q[0] == 'N':
                Q = hex(int(Q))[2:]
                Q += '0' * (len(Q) % 2)
                Q = Q.decode('hex')
            elif self.Q[0] == 'H':
                Q = Q.decode('hex')
            elif self.Q[0] == 'A':
                ## nothing to do - take and append
                pass

            datainput = Q
            datainput += '\0' * (128 - len(Q))

        #log.error("Q %s" % (binascii.hexlify(datainput)))
        return datainput

    def _addPin(self, P=None, P_digest=None):
        datainput = ''
        if self.P:
            if P_digest:
                if len(P) == self.P.digest_size:
                    datainput += P_digest
                elif len(P) == 2 * self.P.digest_size:
                    datainput += P_digest.decode('hex')
                else:
                    raise ValueError, ('Pin/Password digest invalid', P_digest)
            elif P is None:
                raise ValueError, 'Pin/Password missing'
            else:
                datainput = self.P(P).digest()
        #log.error("P %s" % (binascii.hexlify(datainput)))
        return datainput

    @log_with(log)
    def _addSession(self, S):
        datainput = ''
        if self.S:
            if S is None or len(S) > self.S:
                raise ValueError, 'session'

            datainput = S
            datainput += '\0' * (self.S - len(S))
        return datainput

    def _addTimeStr(self, T=None, T_precomputed=None):
        datainput = ''
        if self.T:
            if T_precomputed is not None and is_int(T_precomputed):
                #t = time.gmtime(T_precomputed*60)
                #timestr = time.strftime('%Y-%m-%d:%H:%M:%S',t)
                data = int2beint64(int(T_precomputed))
            elif is_int(T):
                #t = time.gmtime(T)
                #timestr = time.strftime('%Y-%m-%d:%H:%M:%S',t)
                data = int2beint64(int(T / self.T))

            else:
                raise ValueError, 'time format error'
            datainput += data
        #log.error("T %s" % (binascii.hexlify(datainput)))
        return datainput

    def data2hashChallenge(self, data):
        c_type = self.Q[0]
        c_len = self.Q[1]

        challenge_bin = self.hash_algo(data)
        challenge_hex = challenge_bin.hexdigest()

        if c_type == 'A':
            challenge = challenge_hex[:c_len]
        elif c_type == 'H':
            challenge = challenge_hex[:c_len * 2]
        elif c_type == 'N':
            challenge_num = int(challenge_hex[:c_len], 16)
            challenge = unicode(challenge_num)

        if len(challenge) < c_len:
            challenge += '\0' * (c_len - len(challenge))
        challenge = challenge[:c_len]

        return unicode(challenge)

    def data2rawChallenge(self, data):

        c_type = self.Q[0]
        c_len = self.Q[1]

        challenge = ''
        chall = self.data2randomChallenge(data)

        if c_type == 'A':
            for c in data:
                if c.isalnum() and ord(c) < 128:
                    challenge += c

        elif c_type == 'N':
            for c in data:
                if c.isdigit():
                    challenge += c

        elif c_type == 'H':
            for c in data:
                if c.isdigit() or c.lower() in ['a', 'b', 'c', 'd', 'e', 'f']:
                    challenge += c

        for i in range(0, 4):
            if len(challenge) < c_len:
                challenge += '0'

        c_ = len(challenge)
        challenge += chall[c_:]
        challenge = challenge[:c_len]

        return unicode(challenge)

    def data2randomChallenge(self, data):
        '''
            build a random challenge according to the challenge definition
        '''
        alphnum = 'abcdefghijklmnopqrstuvwxyz' + 'ABCDEFGHIJKLMNOPQRSTUVWXYZ' + '0123456789'
        digits = '0123456789'
        hex = digits + 'abcdef'

        challenge = ''

        c_type = self.Q[0]
        c_len = self.Q[1]

        if c_type == 'A':
            for c in range(0, c_len):
                challenge += urandom.choice(alphnum)

        elif c_type == 'N':
            for c in range(0, c_len):
                challenge += urandom.choice(digits)

        elif c_type == 'H':
            for c in range(0, c_len):
                challenge += urandom.choice(hex)

        challenge = challenge[:c_len]
        return unicode(challenge)

    @log_with(log)
    def checkOtp(self, passw, counter, window, ocraChallenge, pin='', options=None, timeshift=0):
        '''
        check the given passw

        :param passw:     the otp to verified
        :param counter:   the start counter from the token
        :param window:    the range, within the counter should be checked
        :param challenge: the ocra challenge, which goes into the otp calculation
        :param pin:       the ocra token pin
        :param options:   support to identifies nonsequential otp verification
        :param timeshif:  for timebased tokens we support time offsets

        :return:          counter of match - otherwise -1
        '''
        ret = -1

        ## std counter for tokens w.o. timer or counter
        start = counter
        end = counter + 1
        step = 1

        ## callculate the start for the timer based tokens
        if self.T is not None:
            ttime = datetime.now()
            ftime = ttime.strftime("%s")
            otime = int(ftime) + timeshift

            step = 1  #self.T

            start = otime - (window * self.T)
            end = otime + (window * self.T)

            ## counter preserves the last access time, so we dont allow lookup in the past
            if start < counter:
                ## support the assynchronous handling of transactions
                if options is not None and options.has_key('transactionid'):
                    pass
                else:
                    start = counter

        ## callculate the start for the counter based tokens
        if self.C is not None:
            start = counter
            end = counter + window
            if options is not None and options.has_key('transactionid'):
                ## in case of a provided transactionid, we scroll back in counter
                ## to support asynchronous transaction verification

                if counter > window / 2:
                    start = counter - window / 2
                else:
                    start = 0
            sdate = datetime.fromtimestamp(start)
            edate = datetime.fromtimestamp(end)
            log.debug('lookup for timerange:  %r - %r '
                       % (sdate, edate))

        ## finally do the check of the otps
        session = ''
        if ocraChallenge.has_key('session'):
            session = ocraChallenge.get('session')

        ## required - will raise exception, if not present
        challenge = ocraChallenge.get('challenge')
        idx = challenge.find(':')
        if idx != -1:
            challenge = challenge[idx + 1:]

        param = {}
        param['Q'] = unicode(challenge)
        param['P'] = unicode(pin)
        param['S'] = unicode(session)

        for count in range(start, end, step):
            if self.C is not None:
                param['C'] = count

            if self.T is not None:
                param['T'] = count

            c_data = self.combineData(**param)
            otp = self.compute(c_data)

            if passw == otp:
                ret = count
                break

        ## support some logging at the end
        if ret == -1:
            if self.T is not None:
                sdate = datetime.fromtimestamp(start)
                edate = datetime.fromtimestamp(end)
                log.info('failed for otp val %r :(exp %r)'
                     ' for timerange:  %r - %r ' % (otp, passw, sdate, edate))
            else:
                log.info('failed for otp val %r :(exp %r)'
                         ' for range: %r - %r' % (otp, passw, start, end))
        return ret


def main():  # pragma: no cover

    import struct

    # ocrasuite   = 'OcraSuite-1:HOTP-SHA256-8:C-QN08-S128-PSHA1'
    ocrasuite = 'OCRA-1:HOTP-SHA256-8:QA08'
    key = '3132333435363738393031323334353637383930313233343536373839303132'.decode('hex')
    pin = '1234'
    session = 'Kontonummer:1234568|BLZ:5675522|Betrag:1234343434,99'
    challenge = '12345678'
    counter = 11

    param = {}
    param['C'] = counter
    param['Q'] = challenge
    param['P'] = pin
    param['S'] = session

    ocra = OcraSuite(ocrasuite)
    data = ocra.combineData(**param)
    otp = ocra.compute(key, data)

    data = struct.pack(">Q", counter)
    print "data %r" % binascii.hexlify(data)
    print "otp %r" % otp


if __name__ == '__main__':

    '''
     devel hook - to be removed later
    '''

    main()


#eof###########################################################################




# -*- coding: utf-8 -*-
#  privacyIDEA is a fork of LinOTP
#  May 08, 2014 Cornelius Kölbel
#  License:  AGPLv3
#  contact:  http://www.privacyidea.org
#
#  Copyright (C) 2010 - 2014 LSE Leading Security Experts GmbH
#  License:  AGPLv3
#  contact:  http://www.linotp.org
#            http://www.lsexperts.de
#            linotp@lsexperts.de
#
# This code is free software; you can redistribute it and/or
# modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
# License as published by the Free Software Foundation; either
# version 3 of the License, or any later version.
#
# This code is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU AFFERO GENERAL PUBLIC LICENSE for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with this program.  If not, see <http://www.gnu.org/licenses/>.
#  
'''
library to do validation
'''


import logging

from sqlalchemy import desc

from privacyidea.lib.user import getUserId

import privacyidea.lib.token
from privacyidea.lib.policy import PolicyClass
from privacyidea.lib.config import get_privacyIDEA_config
from pylons import request, config, tmpl_context as c

from privacyidea.model import Challenge
from privacyidea.model.meta import Session
from privacyidea.lib.config  import getFromConfig
from privacyidea.lib.resolver import getResolverObject
import traceback
from privacyidea.lib.log import log_with

LOG = logging.getLogger(__name__)


@log_with(LOG)
def transform_challenges(challenges):
    '''
    small helper to transfor a set of DB Challenges to a list
    of challenge data as dicts

    :param challenges: list of database challenges

    :return: a list with challenge data dicts
    '''
    channel_list = []
    for challenge in challenges:
        channel_list.append(challenge.get())
    #return channel_list
    return challenges

@log_with(LOG)
def get_challenges(serial=None, transid=None):
    '''
    get_challenges - give all challenges for a given token

    :param serial:   serial of the token
    :param transid:  transaction id, if None, all will be retrieved
    :return:         return a list of challenge dict
    '''
    challenges = []
    if transid is None and serial is None:
        return challenges

    if transid is None:
        db_challenges = Session.query(Challenge)\
            .filter(Challenge.tokenserial == u'' + serial)\
            .order_by(desc(Challenge.id))\
            .all()
    else:
        db_challenges = Session.query(Challenge)\
            .filter(Challenge.transid == transid)\
            .all()

    challenges.extend(db_challenges)
    return challenges

@log_with(LOG)
def create_challenge(token, options=None):
    """
    dedicated method to create a challenge to support the implementation
    of challenge policies in future

    :param options: optional parameters for token specific tokens
                    eg. request a signed challenge
    :return: a tuple of  (boolean, and a dict, which contains the
             {'challenge' : challenge} description)
    """

    ## response dict, describing the challenge reply
    challenge = {}
    ## the allocated db challenge object
    challenge_obj = None
    retry_counter = 0
    reason = None

    id_length = int(getFromConfig('TransactionIdLength', 12))

    while True:
        try:

            transactionid = Challenge.createTransactionId(length=id_length)
            num_challenges = Session.query(Challenge).\
                    filter(Challenge.transid == transactionid).count()

            if num_challenges == 0:
                challenge_obj = Challenge(transid=transactionid,
                                                tokenserial=token.getSerial())
            if challenge_obj is not None:
                break

        except Exception as exce:
            LOG.info("Failed to create Challenge: %r", exce)
            reason = exce

        ## prevent an unlimited loop
        retry_counter = retry_counter + 1
        if retry_counter > 100:
            LOG.info("Failed to create Challenge for %d times: %r -quiting!",
                     retry_counter, reason)
            raise Exception('Failed to create challenge %r' % reason)

    challenges = get_challenges(serial=token.getSerial())

    ## carefully create a new challenge
    try:

        ## we got a challenge object allocated and initialize the challenge
        (res, open_transactionid, message, attributes) = \
                             token.initChallenge(transactionid,
                                                 challenges=challenges,
                                                 options=options)

        if res == False:
            ## if a different transid is returned, this indicates, that there
            ## is already an outstanding challenge we can refere to
            if open_transactionid != transactionid:
                transactionid = open_transactionid

        else:
            ## in case the init was successfull, we preserve no the challenge data
            ## to support the implementation of a blocking based on the previous
            ## stored data
            challenge_obj.setChallenge(message)
            challenge_obj.save()

            (res, message, data, attributes) = \
                        token.createChallenge(transactionid, options=options)

            if res == True:
                ## persist the final challenge data + message
                challenge_obj.setChallenge(message)
                challenge_obj.setData(data)
                challenge_obj.save()
            else:
                transactionid = ''

    except Exception as exce:
        reason = exce
        res = False


    ## if something goes wrong with the challenge, remove it
    if res == False and challenge_obj is not None:
        try:
            LOG.debug("deleting session")
            Session.delete(challenge_obj)
            Session.commit()
        except Exception as exx:
            LOG.debug("deleting session failed: %r" % exx)
            try:
                Session.expunge(challenge_obj)
                Session.commit()
            except Exception as exx:
                LOG.debug("expunge session failed: %r" % exx)

    ## in case that create challenge fails, we must raise this reason
    if reason is not None:
        message = "%r" % reason
        LOG.error("Failed to create or init challenge %r " % reason)
        raise reason

    ## prepare the response for the user
    if transactionid is not None:
        challenge['transactionid'] = transactionid

    if message is not None:
        challenge['message'] = message

    if attributes is not None and type(attributes) == dict:
        challenge.update(attributes)

    return (res, challenge)

@log_with(LOG)
def check_pin(token, passw, user=None, options=None):
    '''
    check the provided pin w.r.t. the policy definition

    :param passw: the to be checked pass
    :param user: if otppin==1, this is the user, which resolver should
                 be checked
    :param options: the optional request parameters

    :return: boolean, if pin matched True
    '''
    res = False
    Policy = PolicyClass(request, config, c,
                         get_privacyIDEA_config())
    pin_policies = Policy.get_pin_policies(user)

    if 1 in pin_policies:
        # We check the Users Password as PIN
        LOG.debug("pin policy=1: checking the users"
                                                    " password as pin")
        if (user is None):
            raise Exception("fail for pin policy == 1 with user = None")

        (uid, _resolver, resolver_class) = getUserId(user)

        r_obj = getResolverObject(resolver_class)
        if  r_obj.checkPass(uid, passw):
            LOG.debug("Successfully authenticated user %r." % uid)
            res = True
        else:
            LOG.info("user %r failed to authenticate." % uid)

    elif 2 in pin_policies:
        # NO PIN should be entered atall
        LOG.debug("pin policy=2: checking no pin")
        if len(passw) == 0:
            res = True
    else:
        # old stuff: We check The fixed OTP PIN
        LOG.debug("pin policy=0: checkin the PIN")
        res = token.checkPin(passw, options=options)

    return res

@log_with(LOG)
def check_otp(token, otpval, options=None):
    '''
    check the otp value

    :param otpval: the to be checked otp value
    :param options: the additional request parameters

    :return: result of the otp check, which is
            the matching otpcounter or -1 if not valid
    '''
    res = -1

    counter = token.getOtpCount()
    window = token.getOtpCountWindow()

    res = token.checkOtp(otpval, counter, window, options=options)
    return res

@log_with(LOG)
def split_pin_otp(token, passw, user=None, options=None):
    '''
    split the pin and the otp fron the given password

    :param passw: the to be splitted password
    :param options: currently not used, but might be forwarded to the
                    token.splitPinPass
    :return: tuple of (split status, pin and otpval)
    '''
    Policy = PolicyClass(request, config, c,
                         get_privacyIDEA_config())
    pin_policies = Policy.get_pin_policies(user)

    policy = 0

    if 1 in pin_policies:
        LOG.debug("pin policy=1: checking the users password as pin")
        # split the passw into password and otp value
        (res, pin, otp) = token.splitPinPass(passw)
        policy = 1
    elif 2 in pin_policies:
        # NO PIN should be entered atall
        LOG.debug("pin policy=2: checking no pin")
        (res, pin, otp) = (0, "", passw)
        policy = 2
    else:
        # old stuff: We check The fixed OTP PIN
        LOG.debug("pin policy=0: checkin the PIN")
        (res, pin, otp) = token.splitPinPass(passw)

    if res != -1:
        res = policy
    return (res, pin, otp)


class ValidateToken(object):
    '''
    class to manage the validation of a token
    '''

    class Context(object):
        '''
        little helper class to prove the interface calls valid
        '''
        def __init__(self):
            '''
            initlize the only api member
            '''
            self.audit = {}


    #@log_with(LOG)
    def __init__(self, token, user=None, context=None):
        '''
        ValidateToken constructor

        :param token: the to checked token
        :param user: the user of the check request of the token user
        :param context: this is used to preserve the context, which is used
                        to not import the global c
        '''
        self.token = token
        self.user = user
        self.pin_policies = None

        ## these lists will be returned as result of the token check
        self.challenge_token = []
        self.pin_matching_token = []
        self.invalid_token = []
        self.valid_token = []

        ## support of context  : c.audit
        if context == None:
            self.context = self.Context()
        else:
            self.context = context

    @log_with(LOG)
    def get_verification_result(self):
        '''
        return the internal result representation of the token verification
        which are a set of list, which stand for the challenge, pinMatching
        or invalid or valid token list

        - the lists are returned as they easily could be joined into the final
          token list, independent of they are empty or contain a token obj

        :return: tuple of token lists
        '''

        return (self.challenge_token, self.pin_matching_token,
                self.invalid_token, self.valid_token)

    #@log_with(LOG)
    def checkToken(self, passw, user, options=None):
        '''
        validate a token against the provided pass

        :raises: "challenge not found",
                 if a state is given and no challenge is found for this
                 challenge id

        :param passw: the password, which could either be a pin, a pin+otp
                       or otp
        :param user: the user which the token belongs to
        :param options: dict with additional request parameters

        :return: tuple of otpcounter and potential reply
        '''
        res = -1
        if options is None:
            options = {}

        ## are there outstanding challenges
        challenges = self.get_challenges(options)

        ## is the request refering to a previous challenge
        if self.token.is_challenge_response(passw, user,
                                             options=options,
                                             challenges=challenges):

            (res, reply) = self.check_challenges(challenges,
                                        user, passw, options=options)

        else:
            ## do the standard check
            (res, reply) = self.check_standard(passw, user, options=options)

        return (res, reply)

    @log_with(LOG)
    def check_challenges(self, challenges, user, passw, options=None):
        '''
        This function checks, if the given response (passw) matches
        any of the open challenges

        to prevent the token author to deal with the database layer, the
        token.checkResponse4Challenge will recieve only the dictionary of the
        challenge data

        :param challenges: the list of database challenges
        :param user: the requesting use
        :param passw: the to password of the request, which must be pin+otp
        :param options: the addtional request parameters
        :return: tuple of otpcount (as result of an internal token.checkOtp)
                 and additional optional reply
        '''
        ## challenge reply will stay None as we are in the challenge response
        ## mode
        reply = None
        matching_challenges = []
        if options == None: options = {}

        otp = passw

        (otpcount, matching_challenges) = self.token.checkResponse4Challenge(
                                            user, otp, options=options,
                                            challenges=challenges)
        if otpcount >= 0:
            self.valid_token.append(self.token)
            if len(self.invalid_token) > 0:
                del self.invalid_token[0]
        else:
            self.invalid_token.append(self.token)

        ## delete all challenges, which belong to the token and
        ## the token could decide on its own, which should be deleted
        ## default is: challenges which are younger than the matching one
        ## are to be deleted

        all_challenges = self.lookup_challenge()
        to_be_deleted = self.token.challenge_janitor(matching_challenges,
                                                      all_challenges)
        self.delete_challenges(to_be_deleted)

        return (otpcount, reply)

    #@log_with(LOG)
    def check_standard(self, passw, user, options=None):
        '''
        do a standard verification, as we are not in a challengeResponse mode

        the upper interfaces expect in the success the otp counter or at
        least 0 if we have a success. A -1 identifies an error

        :param passw: the password, which should be checked
        :param options: dict with additional request parameters

        :return: tuple of matching otpcounter and a potential reply
        '''

        otp_count = -1
        pin_match = False
        reply = None

        ttype = self.token.getType()

        ## fallback eg. in case of check_s, which does not provide a user
        if user is None:
            user = privacyidea.lib.token.get_token_owner(self.token)
            
        Policy = PolicyClass(request, config, c,
                             get_privacyIDEA_config())
        support_challenge_response = Policy.get_auth_challenge_response(user, ttype)

        ## special handling for tokens, who support only challenge modes
        ## like the sms, email or ocra2 token
        challenge_mode_only = False

        mode = self.token.mode
        if type(mode) == list and len(mode) == 1 and mode[0] == "challenge":
            challenge_mode_only = True

        ## the support_challenge_response is overruled, if the token
        ## supports only challenge processing
        if challenge_mode_only == True:
            support_challenge_response = True

        try:
            ## call the token authentication
            (pin_match, otp_count, reply) = self.token.authenticate(passw, user,
                                                                options=options)
        except Exception as exx:
            if (support_challenge_response == True and
                self.token.is_challenge_request(passw, user, options=options)):
                LOG.info("Retry on base of a challenge request:")
                pin_match = False
                otp_count = -1
            else:
                LOG.error("%s" % (traceback.format_exc()))
                raise Exception(exx)

        if otp_count < 0 or pin_match == False:

            if (support_challenge_response == True and
                self.token.isActive() and
                self.token.is_challenge_request(passw, user, options=options)):
                # we are in createChallenge mode
                # fix for #12413:
                # - moved the create_challenge call to the checkTokenList!
                ## after all tokens are processed and only one is challengeing
                # (_res, reply) = create_challenge(self.token, options=options)
                self.challenge_token.append(self.token)

        if len(self.challenge_token) == 0:
            if otp_count >= 0:
                self.valid_token.append(self.token)
            elif pin_match is True:
                self.pin_matching_token.append(self.token)
            else:
                self.invalid_token.append(self.token)

        return (otp_count, reply)

    @log_with(LOG)
    def get_challenges(self, options=None):
        '''
        get all challenges, defined either by the option=state
        or identified by the token serial reference

        :param options: the request options

        :return: a list of challenges
        '''
        challenges = []
        valid_challenges = []

        if (options is not None and
            options.has_key("state") or options.has_key("transactionid")):

            if options.has_key("state"):
                state = options.get('state')
            elif options.has_key("transactionid"):
                state = options.get('transactionid')

            challenges = self.lookup_challenge(state)
            if len(challenges) == 0 and self.token.getType() not in ['ocra']:
                ## if state argument is given, but no open challenge found
                ## this might be a problem, so make a log entry
                LOG.info('no challenge with state %s found for %s'
                            % (state, self.token.getSerial()))

        else:
            challenges = self.lookup_challenge()

        ## now verify that the challenge is valid
        for ch in challenges:
            if self.token.is_challenge_valid(ch):
                valid_challenges.append(ch)

        return valid_challenges

    @log_with(LOG)
    def lookup_challenge(self, state=0):
        '''
        database lookup to find all challenges belonging to a token and or
        if exist with a transaction state

        :param state: the optional parameter identified the state/transactionId

        :return: the list of challenges
        '''

        challenges = []

        if state == 0:
            challenges = Session.query(Challenge).\
              filter(Challenge.tokenserial == self.token.getSerial()).\
              all()
        else:
            challenges = Session.query(Challenge).\
              filter(Challenge.tokenserial == self.token.getSerial()).\
              filter(Challenge.transid == state).\
              all()

        return challenges

    @log_with(LOG)
    def delete_challenges(self, challenges):
        '''
        delete challenges, which match those listed ones

        :param challenges: list of (dict|int|str) challenges
        :return: result of the delete operation
        '''

        challenge_ids = []
        for challenge in challenges:
            if type(challenge) == dict:
                if challenge.has_key('id'):
                    challenge_id = challenge.get('id')
            elif type(challenge) == Challenge:
                challenge_id = challenge.get('id')
            elif type(challenge) in (unicode , str , int):
                challenge_id = challenge

            try:
                challenge_ids.append(int(challenge_id))
            except ValueError:
                ## ignore
                LOG.warning("failed to concert the challengeId %r to int()" %
                            challenge_id)

        res = 1

        # 1. get all challeges which are to be deleted
        # 2. self.token.select_challenges()

        if len(challenge_ids) > 0:

            del_challes = Session.query(Challenge).\
                filter(Challenge.tokenserial == u'' + self.token.getSerial()).\
                filter(Challenge.id.in_(challenge_ids)).all()

            for dell in del_challes:
                Session.delete(dell)
                #pass

        return res

#eof###########################################################################
